package be.iminds.ilabt.jfed.highlevel.stitcher.automated_test;

import be.iminds.ilabt.jfed.call_log_output.CallReport;
import be.iminds.ilabt.jfed.call_log_output.LogOutput;
import be.iminds.ilabt.jfed.highlevel.controller.HighLevelController;
import be.iminds.ilabt.jfed.highlevel.controller.TaskThread;
import be.iminds.ilabt.jfed.highlevel.model.AuthorityList;
import be.iminds.ilabt.jfed.highlevel.model.EasyModel;
import be.iminds.ilabt.jfed.highlevel.model.Slice;
import be.iminds.ilabt.jfed.highlevel.stitcher.ParallelStitcher;
import be.iminds.ilabt.jfed.highlevel.stitcher.StitchingReportFactory;
import be.iminds.ilabt.jfed.log.Logger;
import be.iminds.ilabt.jfed.lowlevel.*;
import be.iminds.ilabt.jfed.lowlevel.api.AbstractGeniAggregateManager;
import be.iminds.ilabt.jfed.lowlevel.api.StitchingComputationService;
import be.iminds.ilabt.jfed.lowlevel.api.test.AbstractStitchingPingTest;
import be.iminds.ilabt.jfed.lowlevel.api.test.CommonAMTest;
import be.iminds.ilabt.jfed.lowlevel.api.test.NodeLoginTester;
import be.iminds.ilabt.jfed.lowlevel.api.user_spec.UserSpec;
import be.iminds.ilabt.jfed.lowlevel.api_wrapper.AggregateManagerWrapper;
import be.iminds.ilabt.jfed.lowlevel.api_wrapper.StatusDetails;
import be.iminds.ilabt.jfed.lowlevel.api_wrapper.impl.AutomaticAggregateManagerWrapper;
import be.iminds.ilabt.jfed.lowlevel.authority.AuthorityListModel;
import be.iminds.ilabt.jfed.lowlevel.authority.SfaAuthority;
import be.iminds.ilabt.jfed.lowlevel.connection.*;
import be.iminds.ilabt.jfed.lowlevel.stitching.StitchingCallData;
import be.iminds.ilabt.jfed.lowlevel.stitching.StitchingDirector;
import be.iminds.ilabt.jfed.preferences.JFedPreferences;
import be.iminds.ilabt.jfed.rspec.model.ModelRspecType;
import be.iminds.ilabt.jfed.rspec.request.geni_rspec_3.*;
import be.iminds.ilabt.jfed.rspec.rspec_source.ManifestRspecSource;
import be.iminds.ilabt.jfed.rspec.rspec_source.RequestRspecSource;
import be.iminds.ilabt.jfed.testing.base.ApiTest;
import be.iminds.ilabt.jfed.testing.base.ApiTestResult;
import be.iminds.ilabt.jfed.util.*;
import ch.ethz.ssh2.Connection;
import ch.ethz.ssh2.Session;
import javafx.collections.ListChangeListener;
import org.slf4j.LoggerFactory;

import javax.inject.Inject;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.Unmarshaller;
import java.io.*;
import java.net.MalformedURLException;
import java.net.URISyntaxException;
import java.security.NoSuchAlgorithmException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * ParallelStitchingPingTest:
 * - uses ParallelStitcher. This is why it needs to be in another module than the automated-test module which contains the other automated tests
 * - create a stitched RSpec, contact SCS and do all needed CreateSlivers.
 * - Then test login on node and connectivity using ping.
 * - finally, Delete everything
 */
public class ParallelStitchingPingTest extends AbstractStitchingPingTest {
    private static final org.slf4j.Logger LOG = LoggerFactory.getLogger(ParallelStitchingPingTest.class);

    private final AuthorityList authorityList;
    private final JFedConnectionProvider connectionProvider;
    private final GeniUserProvider geniUserProvider;
    private final EasyModel easyModel;
    private final TaskThread tt;
    private final HighLevelController highLevelController;
    private final StitchingReportFactory stitchingReportFactory;
    private final AutomaticAggregateManagerWrapper.AutomaticAggregateManagerWrapperFactory automaticAggregateManagerWrapperFactory;

    private ParallelStitcher parallelStitcher;

    private SSHKeyHelper userSshKey;

    @Inject
    public ParallelStitchingPingTest(Logger logger, TargetAuthority testedAuthority, GeniUser user,
                                     JFedConnectionProvider connectionProvider, AuthorityListModel authorityListModel,
                                     JFedPreferences jFedPreferences,
                                     AuthorityList authorityList, JFedConnectionProvider connectionProvider1,
                                     GeniUserProvider geniUserProvider, EasyModel easyModel, TaskThread tt,
                                     HighLevelController highLevelController, StitchingReportFactory stitchingReportFactory,
                                     AutomaticAggregateManagerWrapper.AutomaticAggregateManagerWrapperFactory automaticAggregateManagerWrapperFactory) {
        super(logger, testedAuthority, user, connectionProvider, authorityListModel, jFedPreferences);
        this.authorityList = authorityList;
        this.connectionProvider = connectionProvider1;
        this.geniUserProvider = geniUserProvider;
        this.easyModel = easyModel;
        this.tt = tt;
        this.highLevelController = highLevelController;
        this.stitchingReportFactory = stitchingReportFactory;
        this.automaticAggregateManagerWrapperFactory = automaticAggregateManagerWrapperFactory;
    }

    @Test(hardDepends = {"createSlice", "generateRspec"}, groups = {"init"})
    public void initStitching() throws JFedException, NoSuchAlgorithmException, MalformedURLException {
        userSshKey = new SSHKeyHelper();

        List userSpecs = new ArrayList<UserSpec>();
        UserSpec userSpec =
                new UserSpec(user.getUserUrnString(), userSshKey.getSshPublicKeyString());
        userSpecs.add(userSpec);

        assert scsAuth != null;
        parallelStitcher =
                new ParallelStitcher(tt, easyModel, geniUserProvider,
                        new RequestRspecSource(rspecXmlStr, ModelRspecType.BASIC), connectionProvider,
                        scsAuth, highLevelController, userSpecs, expirationDate.toInstant(), authorityList,
                        jFedPreferences, automaticAggregateManagerWrapperFactory,
                        null /* default time between status checks*/,
                        null /* default ms before giving up on status*/,
                        null /* default ms before retry createsliver after delete*/,
                        logger, stitchingReportFactory);

        director = parallelStitcher.getDirector();

        super.initStitching();
    }

    @Test(hardDepends = {"createSlice", "generateRspec", "initStitching"}, groups = {"init"})
    public void doParallelStitching() throws JFedException, NoSuchAlgorithmException, MalformedURLException, InterruptedException {
        assert rspecXmlStr != null;

        if (parallelStitcher.getStitchingReport().getCurState() != null)
            note("ParallelStitcher is in initial state \""+parallelStitcher.getStitchingReport().getCurState().getName()+"\"");

        final ApiTestResult.ApiTestMethodResult parallelStitchingTestResult = ParallelStitchingPingTest.this.currentTestResult;

        ListChangeListener<LogOutput.LogLine> stateLogLineListener = new ListChangeListener<LogOutput.LogLine>() {
            @Override
            public void onChanged(Change<? extends LogOutput.LogLine> c) {
                while (c.next()) {
                    assert c.wasAdded();
                    assert !c.wasPermutated();
                    assert !c.wasRemoved();
                    assert !c.wasReplaced();
                    assert !c.wasUpdated();
                    for (LogOutput.LogLine logLine : c.getAddedSubList()) {
                        parallelStitchingTestResult.addLogLine(logLine);
                    }
                }
            }
        };
        ListChangeListener<CallReport.State> stateListChangeListener = new ListChangeListener<CallReport.State>() {
                    @Override
                    public void onChanged(Change<? extends CallReport.State> c) {
                        LOG.debug("doParallelStitching() State changed");
                        while (c.next()) {
                            assert c.wasAdded();
                            assert !c.wasPermutated();
                            assert !c.wasRemoved();
                            assert !c.wasReplaced();
                            assert !c.wasUpdated();
                            for (CallReport.State newState : c.getAddedSubList()) {
                                LOG.debug("doParallelStitching() State changed -> added "+newState.getName());
                                parallelStitchingTestResult.addLogLine(LogOutput.LogLineType.NOTE, "ParallelStitcher switched to state \"" + newState.getName() + "\"");
                                synchronized (newState.getLogLines()) {
                                    newState.getLogLines().addListener(stateLogLineListener);
                                    for (LogOutput.LogLine logLine : newState.getLogLines()) {
                                        parallelStitchingTestResult.addLogLine(logLine);
                                    }
                                }
                            }
                        }
                    }
                };

        /* Copy all log lines to the test result */
        if (parallelStitcher.getStitchingReport().getCurState() != null)
            parallelStitcher.getStitchingReport().getCurState().getLogLines().addListener(stateLogLineListener);
        parallelStitcher.getStitchingReport().getStates().addListener(stateListChangeListener);

        be.iminds.ilabt.jfed.highlevel.model.Slice sliceModel = easyModel.getSlice(sliceInfo.urn);
        assertNotNull(sliceModel, "Could not find slice in easymodel: "+sliceInfo.urn);
        parallelStitcher.setUserCredentialAndSlice(sliceInfo.credentials, sliceModel);
        parallelStitcher.start();

        note("Stitching started. Will wait up to 40 minutes for stitching to finish");
        boolean timeout = !parallelStitcher.waitUntilDone(40, TimeUnit.MINUTES);
        if (!timeout)
            note("ParallelStitcher finished. gaveUp="+parallelStitcher.isGaveUp());
        else
            warn("ParallelStitcher took too long.");

        LOG.debug("doParallelStitching() States count when done: "+parallelStitcher.getStitchingReport().getStates().size());
        boolean assertsOn = false;
        assert assertsOn = true; // Intentional side-effect!!!
        if (assertsOn) {
            int expectedLogLineCount = 0;
            int actualLogLineCount = parallelStitchingTestResult.getLogLines().size();
            for (CallReport.State state: parallelStitcher.getStitchingReport().getStates())
                expectedLogLineCount += state.getLogLines().size();
            LOG.debug("doParallelStitching() expectedLogLineCount="+expectedLogLineCount+" actualLogLineCount="+actualLogLineCount+
                    "   (e <= a) =="+(expectedLogLineCount <= actualLogLineCount));
            assert expectedLogLineCount <= actualLogLineCount : "expectedLogLineCount="+expectedLogLineCount+" actualLogLineCount="+actualLogLineCount;
        }

        note("Waiting 2 seconds for everything that might still be running to finish.");
        //dirty, but needed.  ListResources calls otherwise end up in a later test. Also, otherwise manifest might not be up to date.
        Thread.sleep(2000);
        Thread.yield();

        if (timeout && !parallelStitcher.isGaveUp())
            parallelStitcher.giveUp("took too long");
        assertFalse(timeout, "Stitching took too long: gave up");
        assertFalse(parallelStitcher.isGaveUp(), "ParallelStitcher gave up");

        //extract node login info
        ManifestRspecSource manifestRspecSource = sliceModel.getManifestRspec();
        for (SfaAuthority auth : director.getActivelyInvolvedAuthorities())
            if (isNodeAuth(auth)) {
                NodeLoginTester nodeLoginTester = new NodeLoginTester(this, userSshKey);
                boolean parseFoundLoginInfo = nodeLoginTester.parseSshInfoFromGeni3ManifestRspec(manifestRspecSource.getRspecXmlString(), auth, null);
                nodeLoginTesters.put(auth, nodeLoginTester);
            }
    }


    @Test(hardDepends = {"doParallelStitching"}, groups = {"nodelogin"})
    @Override
    public void loginAndPing() throws JFedException, IOException {
        super.loginAndPing();
    }


    @Test(softDepends = {"doParallelStitching", "loginAndPing"}, hardDepends = {"createSlice"}, groups = {"createsliver", "nodelogin"})
    @Override
    public void callDeletes() throws JFedException {
        //delete will already be called in case of failure, but it's never bad to call it twice.
        super.callDeletes();
    }
}
