package be.iminds.ilabt.jfed.experimenter_gui.canvas;

import be.iminds.ilabt.jfed.experimenter_gui.canvas.impl.AddressPoolCanvasNode;
import be.iminds.ilabt.jfed.experimenter_gui.canvas.impl.RspecCanvasLink;
import be.iminds.ilabt.jfed.experimenter_gui.canvas.impl.RspecChannelCanvasNode;
import be.iminds.ilabt.jfed.experimenter_gui.canvas.impl.RspecGTSNode;
import javafx.beans.property.ObjectProperty;
import javafx.beans.property.SimpleObjectProperty;

import javax.annotation.Nonnull;
import java.util.Objects;

/**
 * User: twalcari
 * Date: 11/8/13
 * Time: 12:27 PM
 */
public class SelectionProvider {

    private final ObjectProperty<Object> selectedObject = new SimpleObjectProperty<>(null);

    //////////////////////////////////////////////////////////////////////////////////////////////////////////

    public SelectionProvider() {
    }

    public void clearAllSelections() {
        selectedObject.setValue(null);
    }

    public CanvasNode getSelectedCanvasNode() {
        if (selectedObject.get() instanceof CanvasNode)
            return (CanvasNode) selectedObject.get();
        else
            return null;
    }

    public void setSelectedCanvasNode(CanvasNode value) {
        if (value == null)
            return;

        selectedObject.set(value);
    }

    public ObjectProperty<Object> selectedObjectProperty() {
        return selectedObject;
    }

    public boolean isNodeSelected(@Nonnull CanvasNode item) {
        return Objects.equals(selectedObject.get(), item);
    }

    public RspecCanvasLink getSelectedCanvasLink() {
        if (selectedObject.get() instanceof RspecCanvasLink)
            return (RspecCanvasLink) selectedObject.get();
        else
            return null;
    }

    public void setSelectedCanvasLink(RspecCanvasLink value) {
        if (value == null)
            return;

        selectedObject.set(value);
    }

    public boolean isLinkSelected(@Nonnull RspecCanvasLink link) {
        return Objects.equals(selectedObject.get(), link);
    }

    public RspecCanvasLink.InterfaceLink getSelectedInterfaceLink() {
        if (selectedObject.get() instanceof RspecCanvasLink.InterfaceLink) {
            return (RspecCanvasLink.InterfaceLink) selectedObject.get();
        } else {
            return null;
        }
    }

    public void setSelectedInterfaceLink(RspecCanvasLink.InterfaceLink selectedInterfaceLink) {
        if (selectedInterfaceLink == null)
            return;

        this.selectedObject.setValue(selectedInterfaceLink);
    }

    public RspecChannelCanvasNode getSelectedChannel() {
        if (selectedObject.get() instanceof RspecChannelCanvasNode)
            return (RspecChannelCanvasNode) selectedObject.get();
        else
            return null;
    }

    public void setSelectedChannel(RspecChannelCanvasNode channel) {
        if (channel == null)
            return;

        this.selectedObject.setValue(channel);
    }


    public RspecGTSNode getSelectedGTSNode() {
        if (selectedObject.get() instanceof RspecGTSNode)
            return (RspecGTSNode) selectedObject.get();
        else
            return null;
    }

    public void setSelectedGTSNode(RspecGTSNode gtsNode) {
        if (gtsNode == null)
            return;

        this.selectedObject.setValue(gtsNode);
    }

    public void setSelectedAddressPool(AddressPoolCanvasNode addressPoolCanvasNode) {
        if(addressPoolCanvasNode == null)
            return;
        this.selectedObject.set(addressPoolCanvasNode);
    }

    public AddressPoolCanvasNode getSelectedAddressPool() {
        if (selectedObject.get() instanceof AddressPoolCanvasNode)
            return (AddressPoolCanvasNode) selectedObject.get();
        else
            return null;
    }
}
