package be.iminds.ilabt.jfed.experimenter_gui.editor.bo;

import be.iminds.ilabt.jfed.experimenter_gui.JFedConfiguration;
import be.iminds.ilabt.jfed.util.GeniUrn;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.util.Collections;
import java.util.List;
import java.util.Map;

/**
 * A node description defines a certain class of nodes (e.g. a physical machine, a VM, a node with wireless
 * capabilities).
 * <p>
 * A node description is backed by one or more RspecNodeDescriptions, which define Rspecs of configurations
 * that match the description.
 * <p>
 * <p>
 * User: twalcari
 * Date: 1/10/14
 * Time: 3:38 PM
 */
public class NodeDescription {
    private final String id;
    private final String name;
    private NodeType type;
    private final List<RspecNodeDescription> rspecNodeDescriptions;
    private final Map<GeniUrn, JFedConfiguration.EnabledTestbed> enabledTestbeds;

    public NodeDescription(@Nonnull String id, @Nonnull String name, @Nullable NodeType type,
                           List<RspecNodeDescription> rspecNodeDescriptions,
                           Map<GeniUrn, JFedConfiguration.EnabledTestbed> enabledTestbeds) {
        this.id = id;
        this.name = name;
        this.type = type;
        this.rspecNodeDescriptions = rspecNodeDescriptions;
        this.enabledTestbeds = enabledTestbeds;
    }

    /**
     * A user-readable name for the Node description.
     *
     * @return
     */
    public String getName() {
        return name;
    }

    public NodeType getType() {
        return type;
    }

    /**
     * An ordered list of Rspec Node descriptions. The list defines in which order the various RspecNodeDescriptions
     * have to be queried for availability.
     *
     * @return
     */
    public List<RspecNodeDescription> getRspecNodeDescriptions() {
        return Collections.unmodifiableList(rspecNodeDescriptions);
    }

    public Map<GeniUrn, JFedConfiguration.EnabledTestbed> getEnabledTestbeds() {
        return enabledTestbeds;
    }

    public String getId() {
        return id;
    }

    public enum NodeType {
        RAW_PC("Bare Metal"),
        VM("VMs"),
        PORT("Ports"),
        RAW_ROUTER("Physical router"),
        GATEWAY("Gateway");

        private final String friendlyName;

        NodeType(String friendlyName) {
            this.friendlyName = friendlyName;
        }

        public String getFriendlyName() {
            return friendlyName;
        }
    }

}
