package be.iminds.ilabt.jfed.experimenter_gui.editor.properties;

import be.iminds.ilabt.jfed.experimenter_gui.JFedConfiguration;
import be.iminds.ilabt.jfed.experimenter_gui.editor.ModelRspecEditor;
import be.iminds.ilabt.jfed.experimenter_gui.editor.properties.cmi.CMIManagerFactory;
import be.iminds.ilabt.jfed.experimenter_gui.editor.properties.map.MapDialogFactory;
import be.iminds.ilabt.jfed.experimenter_gui.tasks.TasksFactory;
import be.iminds.ilabt.jfed.experimenter_gui.ui.progressbar.TaskService;
import be.iminds.ilabt.jfed.experimenter_gui.util.ui.FXUtils;
import be.iminds.ilabt.jfed.experimenter_gui.util.ui.StageUtils;
import be.iminds.ilabt.jfed.highlevel.controller.HighLevelController;
import be.iminds.ilabt.jfed.highlevel.model.AuthorityList;
import be.iminds.ilabt.jfed.highlevel.model.EasyOcciModel;
import be.iminds.ilabt.jfed.rspec.model.javafx_impl.FXRspecNode;
import javafx.scene.Scene;
import javafx.stage.Stage;
import javafx.stage.Window;

import javax.inject.Inject;
import javax.inject.Singleton;

/**
 * User: twalcari
 * Date: 12/2/2014
 * Time: 14:45
 */
@Singleton
public class AdaptivePropertiesDialogFactory {

    private final JFedConfiguration config;
    private final AuthorityList authorityList;
    private final CMIManagerFactory cmiManagerFactory;
    private final MapDialogFactory mapDialogFactory;
    private final EasyOcciModel easyOcciModel;
    private final HighLevelController highLevelController;
    private final TasksFactory tasksFactory;
    private final TaskService taskService;

    @Inject
    public AdaptivePropertiesDialogFactory(JFedConfiguration config, AuthorityList authorityList,
                                           CMIManagerFactory cmiManagerFactory, MapDialogFactory mapDialogFactory, EasyOcciModel easyOcciModel,
                                           HighLevelController highLevelController, TasksFactory tasksFactory, TaskService taskService) {
        this.config = config;
        this.authorityList = authorityList;
        this.cmiManagerFactory = cmiManagerFactory;
        this.mapDialogFactory = mapDialogFactory;
        this.easyOcciModel = easyOcciModel;
        this.highLevelController = highLevelController;
        this.tasksFactory = tasksFactory;
        this.taskService = taskService;
    }

    public AdaptivePropertiesDialog createAdaptivePropertiesDialog(ModelRspecEditor modelRspecEditor, FXRspecNode node) {
        return new AdaptivePropertiesDialog(modelRspecEditor, node, config, authorityList, cmiManagerFactory, mapDialogFactory, easyOcciModel, highLevelController, tasksFactory, taskService);
    }

    public void showAdaptivePropertiesDialog(Window parent,
                                             ModelRspecEditor modelRspecEditor, FXRspecNode node) {
        Scene scene = new Scene(createAdaptivePropertiesDialog(modelRspecEditor, node));

        Stage dialogStage = new Stage();
        StageUtils.setJFedStageIcons(dialogStage);
        dialogStage.setScene(scene);

        dialogStage.setTitle("Properties of " + node.getClientId());
        dialogStage.setMinHeight(550);
        dialogStage.setMaxHeight(550);
        dialogStage.setMinWidth(500);
        FXUtils.centerToOwner(parent, dialogStage);

        dialogStage.showAndWait();
    }
}
