package be.iminds.ilabt.jfed.experimenter_gui.editor.properties.cmi;

import be.iminds.ilabt.jfed.experimenter_gui.JFedConfiguration;
import be.iminds.ilabt.jfed.highlevel.model.AuthorityInfo;
import be.iminds.ilabt.jfed.highlevel.model.AuthorityList;
import be.iminds.ilabt.jfed.lowlevel.authority.SfaAuthority;
import be.iminds.ilabt.jfed.rspec.model.javafx_impl.FXRspecNode;
import be.iminds.ilabt.jfed.util.GeniUrn;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.inject.Inject;
import javax.inject.Singleton;
import java.util.Collections;
import java.util.Comparator;

/**
 * This class is responsible for managing the available ComponentManagerInfo of each authority.
 * <p/>
 * User: twalcari
 * Date: 8/28/2014
 * Time: 16:55
 */
@Singleton
public class GlobalCMIManager implements CMIManager {
    private static final Logger LOG = LoggerFactory.getLogger(ComponentManagerInfo.class);

    private final AuthorityList authorityList;
    private final JFedConfiguration config;

    private final ObservableList<ComponentManagerInfo> componentManagerInfos;

    private final ComponentManagerInfoFactory componentManagerInfoFactory;

    @Inject
    public GlobalCMIManager(AuthorityList authorityList, JFedConfiguration config,
                            ComponentManagerInfoFactory componentManagerInfoFactory) {
        this.authorityList = authorityList;
        this.config = config;
        this.componentManagerInfoFactory = componentManagerInfoFactory;
        componentManagerInfos = FXCollections.observableArrayList();

        //fill the testbed-dropdown with all possible items
        for (AuthorityInfo ai : this.authorityList.authorityInfosProperty()) {
            SfaAuthority auth = ai.getSfaAuthority();

            //available
            ComponentManagerInfo cmi = componentManagerInfoFactory.
                    createComponentManagerInfo(ai, this.config.getTestbedDescription(auth.getUrn()));

            cmi.getAvailableResources().updateComponentIds();
            cmi.getAllResources().updateComponentIds();
            componentManagerInfos.add(cmi);
        }

        Collections.sort(componentManagerInfos, Comparator.comparing(cmi -> cmi.getAuthority().getHrn().toLowerCase()));
    }

    @Override
    public ObservableList<ComponentManagerInfo> getComponentManagerInfos() {
        return componentManagerInfos;
    }

    @Override
    public ObservableList<FXRspecNode> filterComponentIds(ComponentManagerInfo cmi, ObservableList<FXRspecNode> nodesList) {
        return nodesList;
    }

//    @Override
//    public ObservableList<FXRspecNode> getAvailableComponentIds(ComponentManagerInfo cmi) {
//        return cmi.getAvailableResources().getComponentIds();
//    }
//
//    @Override
//    public ObservableList<FXRspecNode> getAllComponentIds(ComponentManagerInfo cmi) {
//        return cmi.getAvailableResources().getComponentIds();
//    }


    @Override
    public ComponentManagerInfo getComponentManagerInfo(GeniUrn componentManagerId) {
        for(ComponentManagerInfo cmi : componentManagerInfos){
            if(cmi.getAuthority().getUrn().equals(componentManagerId))
                return cmi;
        }
        return null;
    }


}
