package be.iminds.ilabt.jfed.experimenter_gui.editor.rspec_validation;

import be.iminds.ilabt.jfed.rspec.convert.RspecConverter;
import be.iminds.ilabt.jfed.rspec.model.ModelRspecType;
import be.iminds.ilabt.jfed.rspec.model.RspecFactory;
import be.iminds.ilabt.jfed.rspec.model.RspecFactoryFactory;
import be.iminds.ilabt.jfed.rspec.model.javafx_impl.FXModelRspec;
import be.iminds.ilabt.jfed.rspec.rspec_source.RequestRspecSource;
import javafx.stage.Window;
import org.controlsfx.control.ButtonBar;
import org.controlsfx.control.action.Action;
import org.controlsfx.dialog.DialogAction;
import org.controlsfx.dialog.Dialogs;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * User: twalcari
 * Date: 1/22/2015
 * Time: 17:01
 */
public class IsRequestValidator implements RspecValidator {
    private static final Logger LOG = LoggerFactory.getLogger(IsRequestValidator.class);

    @Override
    public RspecValidationResult validateRSpec(RspecCheckReason reason, RequestRspecSource requestRspecSource, Window dialogOwner) {
        if (requestRspecSource.getModelRspec(ModelRspecType.FX).getType().equals("manifest")) {
            String abortText = "Abort";
            String fixText = "Convert to Request and continue";
            String ignoreText = "Keep as is and continue";

            boolean showIgnore = false;
            switch (reason) {
                case SWITCH_TO_VISUAL_EDITOR: {
                    return null; //no action needed
                }
                case RUN_EXPERIMENT: {
                    abortText = "Abort and continue editing";
                    ignoreText = "Ignore and Run Experiment Anyway";
                    fixText = "Convert to Request and Run Experiment";
                    showIgnore = true;
                    break;
                }
                case CHECK_RSPEC: {
                    abortText = "Keep as is";
                    fixText = "Convert to Request";
                    showIgnore = false;
                    break;
                }
                case OPENED_FILE: {
                    abortText = "Keep as is";
                    fixText = "Convert to Request";
                    showIgnore = false;
                    break;
                }
            }

            Action fix = new DialogAction(fixText, ButtonBar.ButtonType.YES, false, true, true);
            Action ignore = new DialogAction(ignoreText, ButtonBar.ButtonType.CANCEL_CLOSE, true, true, false);
            Action abort = new DialogAction(abortText, ButtonBar.ButtonType.CANCEL_CLOSE, false, true, false);

            Dialogs d = Dialogs.create().owner(dialogOwner)
                    .message("Problem in RSpec:\n" +
                                    "This is a manifest RSpec, and not a request RSpec.\n\n" +
                                    "Do you want to convert the manifest to a request?"
                    )
                    .title("Not a Request RSpec");
            if (showIgnore)
                d = d.actions(fix, ignore, abort);
            else
                d = d.actions(fix, abort);
            Action chosenAction = d.showError();

            if (chosenAction == fix) {
                LOG.info("Converting to Request RSpec");
                RspecConverter converter = new RspecConverter();

                RspecFactory rspecFactory = RspecFactoryFactory.getRspecFactoryInstance(ModelRspecType.FX);

                Action boundNodes = new DialogAction("Keep All Bindings", ButtonBar.ButtonType.YES, false, true, true);
                Action unboundNodes = new DialogAction("Remove Resource Bindings", ButtonBar.ButtonType.YES, false, true, false);
                Action unboundCMs = new DialogAction("Remove all Bindings", ButtonBar.ButtonType.YES, false, true, false);
                Dialogs d2 = Dialogs.create().owner(dialogOwner)
                                    .message("In a manifest, all nodes are bound to a specific resource.\nIn a request, this is optional.\n" +
                                                    "You can also remove the component manager bindings (thus creating an unbound RSpec).\n" +
                                                    "What do you want to do when converting to a request RSpec?\n"
                                    )
                                    .title("Convert To Request Options");

                d2 = d2.actions(boundNodes, unboundNodes, unboundCMs);
                Action chosenBinding = d2.showConfirm();
//                Optional<Action> chosenBinding = d2.showChoices(boundNodes, unboundNodes, unboundCMs);
                boolean keepNodeBindings = chosenBinding == boundNodes;

                FXModelRspec newModel = (FXModelRspec) converter.convertManifestToRequest(requestRspecSource.getModelRspec(ModelRspecType.FX), keepNodeBindings, rspecFactory);
                if (chosenBinding == unboundCMs)
                    newModel = (FXModelRspec) converter.removeComponentManagerBindings(newModel, rspecFactory);

                return new RspecValidationResult(newModel);
            } else if (chosenAction == ignore) {
                return new RspecValidationResult(requestRspecSource, false /* == ignore */);

            } else if (chosenAction == abort || chosenAction == null) {
                return new RspecValidationResult(requestRspecSource, true/* == abort*/);
            }
        }

        return null;
    }
}
