package be.iminds.ilabt.jfed.experimenter_gui.gts_editor;

import be.iminds.ilabt.jfed.groovy_dsl.DSLStreamException;
import be.iminds.ilabt.jfed.gts.GtsDslWriter;
import be.iminds.ilabt.jfed.rspec.model.javafx_impl.FXModelRspec;
import javafx.beans.property.*;

import javax.annotation.Nullable;
import java.io.File;

/**
 * User: twalcari
 * Date: 5/21/2015
 * Time: 11:22
 */
public class EditableGtsDsl {

    /**
     * The file from which this Rspec has been loaded, or where it has been saved to the last time
     */
    @Nullable
    private File file = null;

    /**
     * The name of the experiment. Typically relates to the filename of the rspec
     */
    private final StringProperty name = new SimpleStringProperty();

    private final ReadOnlyObjectWrapper<FXModelRspec> model = new ReadOnlyObjectWrapper<>();

    private final ReadOnlyStringWrapper dsl = new ReadOnlyStringWrapper();

    public EditableGtsDsl(FXModelRspec model, String name, File inputFile) {
        setModel(model);
        this.name.set(name);
        this.file = inputFile;
    }

    /**
     * Creates an empty editable rspec
     */
    public EditableGtsDsl(String name) {
        setModel(new FXModelRspec("request"));
        this.name.set(name);
    }

    public File getFile() {
        return file;
    }

    public void setFile(File file) {
        assert (file != null);
        this.file = file;
    }

    public String getName() {
        return name.get();
    }

    public void setName(String name) {
        this.name.set(name);
    }

    public StringProperty nameProperty() {
        return name;
    }

    public FXModelRspec getModel() {
        return model.get();
    }

    public ReadOnlyObjectProperty<FXModelRspec> modelProperty() {
        return model.getReadOnlyProperty();
    }

    public void setModel(FXModelRspec model) {
        this.model.set(model);
    }

    public void save() throws DSLStreamException {
        GtsDslWriter gtsDslWriter = new GtsDslWriter(name.get(), model.get());

        dsl.set(gtsDslWriter.call());
    }

    public String getDsl() {
        return dsl.get();
    }

    public ReadOnlyStringProperty dslProperty() {
        return dsl.getReadOnlyProperty();
    }
}
