package be.iminds.ilabt.jfed.experimenter_gui.gts_editor;

import be.iminds.ilabt.jfed.experimenter_gui.gts_editor.actions.GTSEditorActionsFactory;
import be.iminds.ilabt.jfed.experimenter_gui.gts_editor.views.EditableGTSCanvasEditorView;
import be.iminds.ilabt.jfed.experimenter_gui.gts_editor.views.GTSViewsFactory;
import be.iminds.ilabt.jfed.groovy_dsl.DSLStreamException;
import be.iminds.ilabt.jfed.ui.javafx.dialogs.JFDialogs;
import javafx.beans.property.SimpleStringProperty;
import javafx.beans.property.StringProperty;
import javafx.scene.control.Alert;
import javafx.scene.layout.BorderPane;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * User: twalcari
 * Date: 5/18/2015
 * Time: 10:37
 */
public class GTSEditor extends BorderPane {
    private static final Logger LOG = LoggerFactory.getLogger(GTSEditor.class);

    private final EditableGtsDsl editableGtsDsl;
    private final EditableGTSCanvasEditorView canvasEditorView;

    private final GTSViewsFactory gtsViewsFactory;
    private final GTSEditorActionsFactory gtsEditorActionsFactory;

    private final StringProperty status = new SimpleStringProperty();

    public GTSEditor(EditableGtsDsl editableGtsDsl, GTSViewsFactory gtsViewsFactory,
                     GTSEditorActionsFactory gtsEditorActionsFactory) {
        this.editableGtsDsl = editableGtsDsl;

        this.gtsViewsFactory = gtsViewsFactory;
        this.gtsEditorActionsFactory = gtsEditorActionsFactory;


        GTSModelEditor modelEditor = new GTSModelEditor(editableGtsDsl);

        this.canvasEditorView = gtsViewsFactory.createEditableGTSCanvasEditorView(modelEditor);

        this.status.bind(canvasEditorView.statusProperty());


        setCenter(canvasEditorView);
    }


    public EditableGTSCanvasEditorView getEditableGTSCanvasEditorView() {
        return canvasEditorView;
    }


    public String getStatus() {
        return status.get();
    }

    public StringProperty statusProperty() {
        return status;
    }

    public void save() {
        try {
            editableGtsDsl.save();

            Alert alert = new Alert(Alert.AlertType.INFORMATION);
            alert.setTitle("GTS DSL saved");
            alert.setContentText("Your GTS DSL was successfully saved.");
            alert.showAndWait();
        } catch (DSLStreamException ex) {
            LOG.error("Error while saving DSL: {}", ex.getMessage(), ex);

            JFDialogs.create()
                    .title("Saving GTS DSL failed")
                    .message("An error occured while saving your GTS DSL")
                    .showException(ex);
        }
    }

    public void export() {
        gtsEditorActionsFactory.createExportGtsDslAction(editableGtsDsl, getScene().getWindow()).call();
    }


    public EditableGtsDsl getEditableGtsDsl() {
        return editableGtsDsl;
    }
}
