package be.iminds.ilabt.jfed.experimenter_gui.gts_editor;

import be.iminds.ilabt.jfed.experimenter_gui.gts_editor.actions.GTSEditorActionsFactory;
import be.iminds.ilabt.jfed.experimenter_gui.gts_editor.ribbon_tabs.GTSTopologyEditorRibbonTab;
import be.iminds.ilabt.jfed.experimenter_gui.gts_editor.views.GTSViewsFactory;
import be.iminds.ilabt.jfed.experimenter_gui.tabs.RibbonEnabled;
import be.iminds.ilabt.jfed.experimenter_gui.tabs.StatusEnabled;
import be.iminds.ilabt.jfed.experimenter_gui.ui.ribbon.RibbonTab;
import be.iminds.ilabt.jfed.highlevel.model.AuthorityList;
import be.iminds.ilabt.jfed.ui.javafx.GlyphUtils;
import be.iminds.ilabt.jfed.ui.javafx.dialogs.JFDialogs;
import javafx.beans.value.ObservableStringValue;
import javafx.scene.control.ContextMenu;
import javafx.scene.control.MenuItem;
import javafx.scene.control.Tab;
import org.controlsfx.glyphfont.FontAwesome;

import java.util.ArrayList;
import java.util.List;

/**
 * User: twalcari
 * Date: 5/18/2015
 * Time: 10:10
 */
public class GTSEditorTab extends Tab implements RibbonEnabled, StatusEnabled {
    private final static List<RibbonTab> ribbonTabs;

    private static GTSTopologyEditorRibbonTab topologyEditorRibbonTab;

    private final EditableGtsDsl editableGtsDsl;
    private final GTSEditor gtsEditor;


    static {
        ribbonTabs = new ArrayList<>();

        topologyEditorRibbonTab = new GTSTopologyEditorRibbonTab();

        ribbonTabs.add(topologyEditorRibbonTab);


    }

    GTSEditorTab(EditableGtsDsl editableGtsDsl, AuthorityList authorityList, GTSViewsFactory gtsViewsFactory, GTSEditorActionsFactory gtsEditorActionsFactory) {
        this.editableGtsDsl = editableGtsDsl;

        textProperty().bind(editableGtsDsl.nameProperty());

        getStyleClass().add("tab-gts-design");
        setGraphic(GlyphUtils.createGlyph(FontAwesome.Glyph.CLOUD));

        ContextMenu contextMenu = new ContextMenu();
        MenuItem changeNameMenuItem = new MenuItem("Change name", GlyphUtils.createGlyph(FontAwesome.Glyph.PENCIL));
        changeNameMenuItem.setOnAction(actionEvent -> showEditNameDialog());
        contextMenu.getItems().add(changeNameMenuItem);
        setContextMenu(contextMenu);


        this.gtsEditor = new GTSEditor(editableGtsDsl, gtsViewsFactory, gtsEditorActionsFactory);
        setContent(gtsEditor);
    }

    private void showEditNameDialog() {
        JFDialogs.create()
                .owner(this.getGtsEditor())
                .message("Enter a new name:")
                .masthead("Change GTS DSL name")
                .title("Change GTS DSL name")
                .showTextInput(editableGtsDsl.getName())
                .filter(x -> !x.isEmpty())
                .ifPresent(editableGtsDsl::setName);
    }



    public List<? extends RibbonTab> getRibbonTabs() {
        return ribbonTabs;
    }

    public RibbonTab getInitialTabToActivate() {
        return topologyEditorRibbonTab;
    }

    public ObservableStringValue statusProperty() {
        return gtsEditor.statusProperty();
    }

    public GTSEditor getGtsEditor() {
        return gtsEditor;
    }
}
