package be.iminds.ilabt.jfed.experimenter_gui.gts_editor.views;

import be.iminds.ilabt.jfed.experimenter_gui.JFedConfiguration;
import be.iminds.ilabt.jfed.experimenter_gui.canvas.AbstractEditableCanvas;
import be.iminds.ilabt.jfed.experimenter_gui.canvas.CanvasNode;
import be.iminds.ilabt.jfed.experimenter_gui.canvas.impl.CanvasNodeFactory;
import be.iminds.ilabt.jfed.experimenter_gui.canvas.impl.RspecCanvasLink;
import be.iminds.ilabt.jfed.experimenter_gui.canvas.impl.RspecCanvasNode;
import be.iminds.ilabt.jfed.experimenter_gui.editor.ModelRspecUtils;
import be.iminds.ilabt.jfed.experimenter_gui.editor.NewCanvasLink;
import be.iminds.ilabt.jfed.experimenter_gui.editor.ToolboxItem;
import be.iminds.ilabt.jfed.experimenter_gui.editor.views.DraggedItemProvider;
import be.iminds.ilabt.jfed.experimenter_gui.gts_editor.GTSModelEditor;
import be.iminds.ilabt.jfed.experimenter_gui.gts_editor.toolbox.GTSHostToolboxItem;
import be.iminds.ilabt.jfed.experimenter_gui.gts_editor.toolbox.GTSPortToolboxItem;
import be.iminds.ilabt.jfed.gts.GtsDslConstants;
import be.iminds.ilabt.jfed.highlevel.model.AuthorityList;
import be.iminds.ilabt.jfed.rspec.model.RspecNode;
import be.iminds.ilabt.jfed.rspec.model.javafx_impl.FXRspecFactory;
import be.iminds.ilabt.jfed.rspec.model.javafx_impl.FXRspecLink;
import be.iminds.ilabt.jfed.rspec.model.javafx_impl.FXRspecNode;
import be.iminds.ilabt.jfed.ui.javafx.GlyphUtils;
import be.iminds.ilabt.jfed.util.GeniUrn;
import javafx.geometry.Point2D;
import javafx.scene.control.Alert;
import javafx.scene.control.ChoiceDialog;
import javafx.scene.control.ContextMenu;
import javafx.scene.control.MenuItem;
import javafx.scene.input.MouseButton;
import javafx.scene.input.MouseEvent;
import org.controlsfx.glyphfont.FontAwesome;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * User: twalcari
 * Date: 5/21/2015
 * Time: 10:28
 */
public class EditableGTSCanvas extends AbstractEditableCanvas {
    private static final Logger LOG = LoggerFactory.getLogger(EditableGTSCanvas.class);
    private final GTSModelEditor modelEditor;
    private final FXRspecFactory rspecFactory = FXRspecFactory.getInstance();

    private final ContextMenu nodeContextMenu;
    private final ContextMenu linkContextMenu;

    public EditableGTSCanvas(GTSModelEditor modelEditor, JFedConfiguration config, CanvasNodeFactory canvasNodeFactory,
                             DraggedItemProvider draggedItemProvider, AuthorityList authorityList) {
        super(modelEditor.getModelRspec(), config, canvasNodeFactory, draggedItemProvider, authorityList);
        this.modelEditor = modelEditor;

        // create context menus
        this.nodeContextMenu = new ContextMenu();

        MenuItem configureItem = new MenuItem("Configure node", GlyphUtils.createSmallGlyph(FontAwesome.Glyph.PENCIL));
        configureItem.setOnAction(actionEvent -> {
            assert (getSelectionProvider().getSelectedCanvasNode() instanceof RspecCanvasNode);
            RspecCanvasNode rspecCanvasNode = (RspecCanvasNode) getSelectionProvider().getSelectedCanvasNode();

            onConfigureNodePressed(rspecCanvasNode);
        });
        nodeContextMenu.getItems().add(configureItem);

        MenuItem deleteItem = new MenuItem("Delete", GlyphUtils.createSmallGlyph(FontAwesome.Glyph.TIMES));
        deleteItem.setOnAction(actionEvent -> {
            assert (getSelectionProvider().getSelectedCanvasNode() instanceof RspecCanvasNode);
            RspecCanvasNode rspecCanvasNode = (RspecCanvasNode) getSelectionProvider().getSelectedCanvasNode();

            onDeleteNodePressed(rspecCanvasNode);
        });
        nodeContextMenu.getItems().add(deleteItem);

        //contextMenu for links
        this.linkContextMenu = new ContextMenu();

        // was disabled as FXRspecLink.LinkSetting is currently not supported in practice, but it seems to work for VM's now
        MenuItem configureLinkItem =
                new MenuItem("Configure link", GlyphUtils.createSmallGlyph(FontAwesome.Glyph.PENCIL));
        configureLinkItem.setOnAction(actionEvent -> {
            RspecCanvasLink rspecCanvasLink = getSelectionProvider().getSelectedCanvasLink();

            onConfigureLinkPressed(rspecCanvasLink.getRspecLink());
        });
        linkContextMenu.getItems().add(configureLinkItem);

        MenuItem linkDeleteItem = new MenuItem("Delete", GlyphUtils.createSmallGlyph(FontAwesome.Glyph.TIMES));
        linkDeleteItem.setOnAction(actionEvent -> {
            RspecCanvasLink rspecCanvasLink = getSelectionProvider().getSelectedCanvasLink();

            onDeleteLinkPressed(rspecCanvasLink);
        });
        linkContextMenu.getItems().add(linkDeleteItem);


    }

    private void onConfigureLinkPressed(FXRspecLink rspecLink) {
        new Alert(Alert.AlertType.WARNING, "Not implemented yet").showAndWait();
    }

    private void onConfigureNodePressed(RspecCanvasNode rspecCanvasNode) {
        if (rspecCanvasNode.getRspecNode().getComponentManagerId().equals(GtsDslConstants.GTS_URN_PORT_CM)) {
            new Alert(Alert.AlertType.WARNING, "No settings for port yet.").showAndWait();
            return;
        }

        GeniUrn[] locations = new GeniUrn[]{GtsDslConstants.GTS_URN_LOCATION_BRA,
                                            GtsDslConstants.GTS_URN_LOCATION_CPH,
                                            GtsDslConstants.GTS_URN_LOCATION_LJU};


        ChoiceDialog<GeniUrn> dialog = new ChoiceDialog<>(rspecCanvasNode.getRspecNode().getComponentId(), locations);

        dialog.setContentText("Select the location for this host:");
        dialog.showAndWait().ifPresent(urn -> {
            rspecCanvasNode.getRspecNode().setComponentId(urn);
        });

    }

    @Override
    protected void deleteSelectedItem() {
        if (getSelectionProvider().getSelectedCanvasNode() != null) {
            CanvasNode canvasNode = getSelectionProvider().getSelectedCanvasNode();
            if (canvasNode instanceof RspecCanvasNode) {
                onDeleteNodePressed((RspecCanvasNode) canvasNode);
            }
        } else if (getSelectionProvider().getSelectedCanvasLink() != null) {
            RspecCanvasLink rspecCanvasLink = getSelectionProvider().getSelectedCanvasLink();
            onDeleteLinkPressed(rspecCanvasLink);
        }
    }

    private void onDeleteNodePressed(RspecCanvasNode rspecCanvasNode) {
        model.deleteNode(rspecCanvasNode.getRspecNode());
        if (getSelectionProvider().getSelectedCanvasNode() == rspecCanvasNode) {
            getSelectionProvider().clearAllSelections();
        }

    }

    private void onDeleteLinkPressed(RspecCanvasLink rspecCanvasLink) {
        model.deleteLink(rspecCanvasLink.getRspecLink());
        if (getSelectionProvider().getSelectedCanvasLink() == rspecCanvasLink) {
            getSelectionProvider().clearAllSelections();
        }
    }

    @Override
    protected boolean processNewLinkRequest(NewCanvasLink newCanvasLink) {
        LOG.trace("Received new link request between {} or {} AND {} {}", newCanvasLink.getOriginNode(), newCanvasLink.getOriginLink(),
                newCanvasLink.getTargetNode(), newCanvasLink.getTargetLink());

        //delegate the real link-creation to ModelRspecEditor
        LOG.trace("Received link request is OK!");

        if (newCanvasLink.getOriginNode() != null && newCanvasLink.getTargetNode() != null) {
            FXRspecLink newLink = modelEditor.addLink(newCanvasLink.getOriginNode().getRspecNode(),
                    newCanvasLink.getTargetNode().getRspecNode());

            setStatus("Created a new link '" + newLink.getClientId() + "'");
            return true;

        } else {

            setStatus("Can only create links between two nodes!");
            return false;


        }
    }

    @Override
    protected void addCanvasNodeFromToolboxItem(ToolboxItem item, Point2D location) {

        if (item instanceof GTSHostToolboxItem) {
            modelEditor.addHost(location);

        } else if (item instanceof GTSPortToolboxItem) {
            modelEditor.addPort(location);

        } else {
            throw new RuntimeException("Unexpected type of toolboxItem: " + item.getClass().getName());
        }
    }

    @Override
    protected RspecCanvasNode addCopyOfNodeToCanvas(RspecCanvasNode originalRspecCanvasNode) {

        FXRspecNode newRspecNode =
                rspecFactory.copyNode(model, originalRspecCanvasNode.getRspecNode(), RspecNode.InterfaceCopyMethod.NO_COPY);


        newRspecNode.setClientId(model.nextNodeName());
        newRspecNode.setEditorX(newRspecNode.getEditorX() + EDITOR_LOCATION_SHIFT);
        newRspecNode.setEditorY(newRspecNode.getEditorY() + EDITOR_LOCATION_SHIFT);

        model.getNodes().add(newRspecNode);

        return getRspecCanvasNode(newRspecNode);
    }

    public void arrangeNodesOnCircle() {
        arrangeNodesOnCircle(600, 400);
    }

    /**
     * This method arranges all the nodes on an ellipse of the given width and height
     */
    public void arrangeNodesOnCircle(double width, double height) {
        setZoom(1);
        ModelRspecUtils.arrangeNodesOnCircle(modelEditor.getModelRspec(), width, height);
    }

    @Override
    protected void canvasItemOnMouseClicked(MouseEvent mouseEvent) {
        super.canvasItemOnMouseClicked(mouseEvent);

        if (mouseEvent.getSource() instanceof RspecCanvasNode) {
            RspecCanvasNode node = (RspecCanvasNode) mouseEvent.getSource();
            getSelectionProvider().setSelectedCanvasNode(node);

            if (mouseEvent.getButton() == MouseButton.PRIMARY && mouseEvent.getClickCount() == 2) {
                onConfigureNodePressed(node);

            } else if (mouseEvent.getButton() != MouseButton.PRIMARY || mouseEvent.isControlDown()) {
                //show context menu
                nodeContextMenu.show(node, mouseEvent.getScreenX(), mouseEvent.getScreenY());

            }
        }

        if (mouseEvent.getSource() instanceof RspecCanvasLink.LinkCenter) {
            RspecCanvasLink link =
                    ((RspecCanvasLink.LinkCenter) mouseEvent.getSource()).getRspecCanvasLink();


            getSelectionProvider().setSelectedCanvasLink(link);
            assert (getSelectionProvider().getSelectedCanvasLink() != null);

            if (mouseEvent.getButton() == MouseButton.PRIMARY && mouseEvent.getClickCount() == 2) {
                onConfigureLinkPressed(link.getRspecLink());
            } else if (mouseEvent.getButton() != MouseButton.PRIMARY || mouseEvent.isControlDown()) {
                //show context menu
                linkContextMenu.show(getSelectionProvider().getSelectedCanvasLink().getLinkCenter(),
                        mouseEvent.getScreenX(), mouseEvent.getScreenY());
            }
        }
    }
}
