package be.iminds.ilabt.jfed.experimenter_gui.health;

import be.iminds.ilabt.jfed.experimenter_gui.util.GeniUrnTypeAdapter;
import be.iminds.ilabt.jfed.experimenter_gui.util.IntegerTypeAdapter;
import be.iminds.ilabt.jfed.util.GeniUrn;
import be.iminds.ilabt.jfed.util.IOUtils;
import com.fatboyindustrial.gsonjavatime.Converters;
import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.google.gson.reflect.TypeToken;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.inject.Inject;
import java.io.IOException;
import java.lang.reflect.Type;
import java.net.URL;
import java.nio.charset.Charset;
import java.util.Map;
import java.util.concurrent.Callable;

/**
 * User: twalcari
 * Date: 8/20/2015
 * Time: 11:35
 */
public class HealthServiceClient implements Callable<Map<GeniUrn, HealthStatus>> {
    private static final Logger LOG = LoggerFactory.getLogger(HealthServiceClient.class);

    private static final String BASE_URL = "https://flsmonitor.fed4fire.eu/service/";
    //private static final String BASE_URL = "http://localhost:8080/";

    private static final Type HEALTH_OVERVIEW_TYPE = new TypeToken<Map<GeniUrn, HealthStatus>>() {
    }.getType();

    private final Gson gson;


    @Inject
    public HealthServiceClient() {
        GsonBuilder builder = new GsonBuilder();
        builder.registerTypeAdapter(Integer.class, new IntegerTypeAdapter());
        builder.registerTypeAdapter(GeniUrn.class, new GeniUrnTypeAdapter());
        Converters.registerZonedDateTime(builder);
        gson = builder.create();
    }


    @Override
    public Map<GeniUrn, HealthStatus> call() {

        int retries = 3;

        while (retries-- > 0) {
            try {

                return gson.fromJson(IOUtils.urlToString(new URL(BASE_URL + "?names=urn"), Charset.defaultCharset()),
                        HEALTH_OVERVIEW_TYPE);

            } catch (IOException ex) {
                LOG.error("Error while fetching health status", ex);

            }
        }
        return null;
    }
}
