package be.iminds.ilabt.jfed.experimenter_gui.slice;

import be.iminds.ilabt.jfed.highlevel.model.Slice;
import be.iminds.ilabt.jfed.highlevel.model.Sliver;
import be.iminds.ilabt.jfed.lowlevel.api_wrapper.StatusDetails;
import be.iminds.ilabt.jfed.lowlevel.authority.SfaAuthority;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.time.Instant;

/**
 * User: twalcari
 * Date: 8/14/2015
 * Time: 10:38
 */
public class NextExperimentExpiration {
    private static final Logger LOG = LoggerFactory.getLogger(NextExperimentExpiration.class);

    private final Sliver firstExpiringSliver;
    private final Instant firstExpirationTime;

    private final boolean hasPartExpired;


    public NextExperimentExpiration(@Nonnull Slice slice) {

        Sliver firstExpiringSliver = null;
        Instant firstExpirationTime = slice.getExpirationDate();
        boolean hasPartExpired = firstExpirationTime != null && firstExpirationTime.isBefore(Instant.now());


        for (Sliver sliver : slice.getSlivers()) {
            if(sliver.getStatus().getGlobalStatus() == StatusDetails.SliverStatus.UNALLOCATED) {
                LOG.trace("Skipping sliver {}, as status is UNALLOCATED", sliver.getUrn());
                continue; //old slivers that don't exist anymore, should not be taken into account
            }

            boolean hasNodes =  sliver.getNodes() != null && !sliver.getNodes().isEmpty();
            boolean hasLinks =  sliver.getLinks() != null && !sliver.getLinks().isEmpty();
            if (!hasNodes && hasLinks) {
                LOG.trace("Skipping sliver {}, as it has only links", sliver.getUrn());
                continue;
            }

            if (sliver.getExpirationDate() == null) {
                LOG.trace("Skipping sliver {}, as it has no expiration date", sliver.getUrn());
                continue;
            }

            if (sliver.getExpirationDate().isBefore(Instant.now())) {
                hasPartExpired = true;
            } else if (firstExpirationTime == null || sliver.getExpirationDate().isBefore(firstExpirationTime)) {
                firstExpiringSliver = sliver;
                firstExpirationTime = sliver.getExpirationDate();
            }
        }

        this.firstExpirationTime = firstExpirationTime;
        this.firstExpiringSliver = firstExpiringSliver;
        this.hasPartExpired = hasPartExpired;

        LOG.debug("Next expiration time was determined to be {} (source: {})", firstExpirationTime,
                firstExpiringSliver != null ?  firstExpiringSliver.getUrn() : "slice");
    }

    @Nullable
    public Instant getFirstExpirationTime() {
        return firstExpirationTime;
    }

    @Nullable
    public Sliver getFirstExpiringSliver() {
        return firstExpiringSliver;
    }

    public SfaAuthority getFirstExpiringAuthority(){
        return firstExpiringSliver != null ? firstExpiringSliver.getAuthority() : null;
    }

    public boolean hasPartExpired() {
        return hasPartExpired;
    }
}
