package be.iminds.ilabt.jfed.experimenter_gui.slice.tasks;

import be.iminds.ilabt.jfed.experimenter_gui.slice.Experiment;
import be.iminds.ilabt.jfed.experimenter_gui.slice.progress.ProgressItem;
import be.iminds.ilabt.jfed.experimenter_gui.ui.status.TaskStatusIndicator;
import be.iminds.ilabt.jfed.highlevel.controller.TaskExecution;
import javafx.application.Platform;
import javafx.concurrent.Task;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;

/**
 * User: twalcari
 * Date: 2/2/2015
 * Time: 10:29
 */
public abstract class ExperimentTask extends Task<TaskStatusIndicator.Status> implements ProgressItemProvider {
    private static final Logger LOG = LoggerFactory.getLogger(ExperimentTask.class);

    protected final Experiment experiment;
    private final ProgressItem progressItem;

    protected final List<TaskExecutionFinishedListener> taskExecutionFinishedListeners = new ArrayList<>();
    private final List<ExperimentTaskFinishedCallback> finishedCallbacks = new ArrayList<>();

    public ExperimentTask(Experiment experiment) {
        this.experiment = experiment;

        this.progressItem = new ProgressItem();
        this.progressItem.textProperty().bind(titleProperty());
        this.progressItem.statusProperty().bind(messageProperty());
    }

    @Override
    protected final TaskStatusIndicator.Status call() throws Exception {
        setProgressStatus(TaskStatusIndicator.Status.BUSY);

        TaskStatusIndicator.Status finalStatus = executeTask();

        assert (finalStatus == TaskStatusIndicator.Status.SUCCESS ||
                finalStatus == TaskStatusIndicator.Status.WARNING ||
                finalStatus == TaskStatusIndicator.Status.FAILED) : "Unexpected final status";

        setProgressStatus(finalStatus);

        for (ExperimentTaskFinishedCallback callback : finishedCallbacks)
            callback.onExperimentTaskFinished(finalStatus);

        return finalStatus;
    }

    private void setProgressStatus(final TaskStatusIndicator.Status newStatus) {
        Runnable setNewStatusRunnable = () -> {
            progressItem.setProgress(newStatus);
        };

        if (Platform.isFxApplicationThread()) {
            setNewStatusRunnable.run();
        } else {
            Platform.runLater(setNewStatusRunnable);
        }
    }


    protected abstract TaskStatusIndicator.Status executeTask();

    @Override
    public ProgressItem getProgressItem() {
        return progressItem;
    }

    public Experiment getExperiment() {
        return experiment;
    }

    public void addTaskFinishedCallback(ExperimentTaskFinishedCallback callback) {
        finishedCallbacks.add(callback);
    }

    protected void registerFinishedTaskExecution(be.iminds.ilabt.jfed.highlevel.controller.Task task, TaskExecution taskExecution) {
        taskExecutionFinishedListeners.forEach(listener -> listener.onTaskExecutionFinished(task, taskExecution));
    }

    public void addTaskExecutionsListener(TaskExecutionFinishedListener listener) {
        taskExecutionFinishedListeners.add(listener);
    }

    protected void waitForTaskExecutionToComplete(be.iminds.ilabt.jfed.highlevel.controller.Task task, TaskExecution taskExecution) {
        while (!taskExecution.isCompleted()) {
            try {
                Thread.sleep(100);
            } catch (InterruptedException ignored) {
            }
        }

        registerFinishedTaskExecution(task, taskExecution);
    }

    @Override
    protected void failed() {
        LOG.error("An uncaught exception caused ExperimentTask " + getClass().getSimpleName() + " to fail!", getException());

    }
}
