package be.iminds.ilabt.jfed.experimenter_gui.slice.tasks;

import be.iminds.ilabt.jfed.experimenter_gui.bugreporting.BugReportDialogFactory;
import be.iminds.ilabt.jfed.experimenter_gui.slice.Experiment;
import be.iminds.ilabt.jfed.experimenter_gui.slice.ExperimentController;
import be.iminds.ilabt.jfed.experimenter_gui.util.ProxyServiceUtil;
import be.iminds.ilabt.jfed.highlevel.controller.HighLevelController;
import be.iminds.ilabt.jfed.highlevel.model.Sliver;
import be.iminds.ilabt.jfed.lowlevel.GeniUserProvider;
import be.iminds.ilabt.jfed.lowlevel.api.user_spec.UserSpec;
import be.iminds.ilabt.jfed.lowlevel.api_wrapper.impl.AutomaticAggregateManagerWrapper;
import be.iminds.ilabt.jfed.lowlevel.authority.AuthorityFinder;
import be.iminds.ilabt.jfed.lowlevel.authority.AuthorityListModel;
import be.iminds.ilabt.jfed.lowlevel.authority.SfaAuthority;
import be.iminds.ilabt.jfed.preferences.JFedGuiPreferences;
import be.iminds.ilabt.jfed.rspec.model.javafx_impl.FXRspecLink;
import be.iminds.ilabt.jfed.rspec.model.javafx_impl.FXRspecNode;
import be.iminds.ilabt.jfed.util.GeniUrn;
import be.iminds.ilabt.jfed.util.ProxyPreferencesManager;
import javafx.application.HostServices;

import javax.inject.Inject;
import javax.inject.Provider;
import javax.inject.Singleton;
import java.time.Instant;
import java.util.Collection;
import java.util.List;

/**
 * User: twalcari
 * Date: 2/2/2015
 * Time: 15:07
 */
@Singleton
public class ExperimentTasksFactory {

    private final HighLevelController hlc;
    private final AuthorityListModel authorityListModel;
    private final AuthorityFinder authorityFinder;
    private final GeniUserProvider geniUserProvider;
    private final JFedGuiPreferences jFedPreferences;
    private final ProxyPreferencesManager proxyPreferencesManager;
    private final AutomaticAggregateManagerWrapper.AutomaticAggregateManagerWrapperFactory automaticAggregateManagerWrapperFactory;

    private final Provider<HostServices> hostServicesProvider;
    private final BugReportDialogFactory bugReportDialogFactory;
    private final ProxyServiceUtil proxyServiceUtil;

    @Inject
    public ExperimentTasksFactory(HighLevelController hlc,
                                  AuthorityListModel authorityListModel,
                                  AuthorityFinder authorityFinder,
                                  GeniUserProvider geniUserProvider, JFedGuiPreferences jFedPreferences,
                                  ProxyPreferencesManager proxyPreferencesManager,
                                  AutomaticAggregateManagerWrapper.AutomaticAggregateManagerWrapperFactory automaticAggregateManagerWrapperFactory,
                                  Provider<HostServices> hostServicesProvider,
                                  BugReportDialogFactory bugReportDialogFactory,
                                  ProxyServiceUtil proxyServiceUtil) {
        this.hlc = hlc;
        this.authorityListModel = authorityListModel;
        this.authorityFinder = authorityFinder;
        this.geniUserProvider = geniUserProvider;
        this.jFedPreferences = jFedPreferences;
        this.proxyPreferencesManager = proxyPreferencesManager;
        this.automaticAggregateManagerWrapperFactory = automaticAggregateManagerWrapperFactory;
        this.hostServicesProvider = hostServicesProvider;
        this.bugReportDialogFactory = bugReportDialogFactory;
        this.proxyServiceUtil = proxyServiceUtil;
    }

    public CreateDiskImageTask createCreateDiskImageTask(ExperimentController experimentController, SfaAuthority authority, FXRspecNode node, String imageName, boolean global, boolean updatePrepare) {
        return new CreateDiskImageTask(experimentController, authority, node, imageName, global, updatePrepare, hlc, this);
    }


    public StopExperimentTask createStopExperimentTask(Experiment experiment) {
        return new StopExperimentTask(experiment, hlc);
    }

    public RegisterExperimentTask createRegisterExperimentTask(Experiment experiment) {
        return new RegisterExperimentTask(experiment, hlc);
    }

    public RestoreSliceAndSliversTaskGenerator createRestoreSliversTaskGenerator(ExperimentController experimentController) {
        return new RestoreSliceAndSliversTaskGenerator(experimentController, hlc, authorityListModel, authorityFinder);
    }

    public InitializeSliversWithStitchingTaskGenerator createInitializeSliversWithStitchTaskGenerator(ExperimentController experimentController) {
        return new InitializeSliversWithStitchingTaskGenerator(experimentController, hlc, geniUserProvider, authorityListModel, jFedPreferences, this);
    }

    public InitializeSliversWithoutStitchingTaskGenerator createInitializeSliversWithoutStitchTaskGenerator(ExperimentController experimentController) {
        return new InitializeSliversWithoutStitchingTaskGenerator(experimentController, hlc, this, jFedPreferences, hostServicesProvider, bugReportDialogFactory);
    }

    public AuthorityConnectivityTestTask createAuthorityConnectivityTestTask(Experiment experiment, SfaAuthority authority, Collection<FXRspecNode> nodesToTest) {
        return new AuthorityConnectivityTestTask(experiment, authority, nodesToTest, geniUserProvider, jFedPreferences, proxyPreferencesManager, hostServicesProvider, bugReportDialogFactory, proxyServiceUtil);
    }

    public TestConnectivityTaskGenerator createTestConnectivityTaskGenerator(ExperimentController experimentController) {
        return new TestConnectivityTaskGenerator(experimentController, this, jFedPreferences, hostServicesProvider, bugReportDialogFactory);
    }

    public StatusUntilReadyTask createStatusUntilReadyTask(Experiment experiment, SfaAuthority authority) {
        return new StatusUntilReadyTask(experiment, authority, hlc);
    }


    public StatusUntilReadyTask createStatusUntilReadyTask(Experiment experiment, SfaAuthority authority, int initialWaitMillis) {
        return new StatusUntilReadyTask(experiment, authority, initialWaitMillis, hlc);
    }

    public ReloadOSTask createReloadOSTask(ExperimentController experimentController, Sliver sliver) {
        return new ReloadOSTask(experimentController, sliver, hlc, this);
    }

    public RebootSliverTask createRebootSliverTask(ExperimentController experimentController, Sliver sliver) {
        return new RebootSliverTask(experimentController, sliver, hlc, this);
    }

    public RebootSliverTask createRebootSliverTask(ExperimentController experimentController, SfaAuthority authority) {
        return new RebootSliverTask(experimentController, authority, hlc, this);
    }

    public EditSshKeysTaskGenerator createEditSshKeysTaskGenerator(ExperimentController experimentController, List<UserSpec> userspec) {
        return new EditSshKeysTaskGenerator(experimentController, userspec, hlc, automaticAggregateManagerWrapperFactory, this);

    }

    public OpenConsoleTask createOpenConsoleTask(Experiment experiment, Sliver sliver) {
        return new OpenConsoleTask(experiment, sliver, hlc);
    }

    public UpdateExperimentTask createUpdateExperimentTask(Experiment experiment) {
        return new UpdateExperimentTask(experiment, hlc);
    }

    public RenewSliceTask createRenewSliceTask(Experiment experiment, Instant newExpirationTime) {
        return new RenewSliceTask(experiment, newExpirationTime, hlc);
    }

    public RenewSliversAtAuthoritiesTask createRenewSliversAtAuthoritiesTask(Experiment experiment, Instant newExpirationDate, Collection<SfaAuthority> authorities) {
        return new RenewSliversAtAuthoritiesTask(experiment, newExpirationDate, authorities, hlc);
    }

    public ShareSliceTaskGenerator createShareSliceTasksFactory(ExperimentController experimentController, List<GeniUrn> userUrns, boolean addUserSshKeys) {
        return new ShareSliceTaskGenerator(experimentController, userUrns, addUserSshKeys, this, hlc);
    }

    public CheckExpirationTaskGenerator createCheckExpirationTaskGenerator(ExperimentController experimentController) {
        return new CheckExpirationTaskGenerator(experimentController, hlc);
    }

    public ReserveSliversTaskGenerator createReserveSliversTaskGenerator(ExperimentController experimentController) {
        return new ReserveSliversTaskGenerator(experimentController, hlc, this);
    }

    public FetchSliceMemberSshKeysTask createFetchSliceMemberSshKeysTask(ExperimentController experimentController){
        return new FetchSliceMemberSshKeysTask(experimentController.getExperiment(), hlc);
    }

    public ShareLanTask createShareLanTask(ExperimentController experimentController, FXRspecLink link, String sharedLanName, boolean share) {
        return new ShareLanTask(experimentController, link, sharedLanName, share, hlc, authorityListModel, authorityFinder);
    }

    public StopExperimentAtAuthorityTask createStopExperimentAtAuthorityTask(Experiment experiment, SfaAuthority authority) {
        return new StopExperimentAtAuthorityTask(experiment, authority, hlc);
    }
}
