package be.iminds.ilabt.jfed.experimenter_gui.slice.tasks;

import be.iminds.ilabt.jfed.experimenter_gui.slice.Experiment;
import be.iminds.ilabt.jfed.experimenter_gui.slice.SliceState;
import be.iminds.ilabt.jfed.highlevel.controller.HighLevelController;
import be.iminds.ilabt.jfed.highlevel.controller.Task;
import be.iminds.ilabt.jfed.highlevel.controller.TaskExecution;
import be.iminds.ilabt.jfed.lowlevel.JFedException;
import be.iminds.ilabt.jfed.ui.javafx.dialogs.JFDialogs;
import javafx.application.Platform;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * User: twalcari
 * Date: 2/2/2015
 * Time: 17:53
 */
public class RegisterExperimentTask extends SingleCallExperimentTask {
    private static final Logger LOG = LoggerFactory.getLogger(RegisterExperimentTask.class);
    private final HighLevelController.CreateSliceTask createSliceTask;

    RegisterExperimentTask(Experiment experiment, HighLevelController hlc) {
        super(experiment, hlc);

        assert (experiment.getSlice() == null);
        assert (experiment.getRequestedEndTime() != null);

        this.createSliceTask =
                hlc.createSlice(experiment.getName(), experiment.getSubAuthName(), experiment.getRequestedEndTime());

        updateTitle("Register experiment " + experiment.getName());
    }

    @Override
    public Task createTask() {
        return createSliceTask;
    }

    @Override
    public void onTaskFinished(TaskExecution taskExecution) {
        if (createSliceTask.getSlice() == null) {
            LOG.error("Create Slice task failed. (createSliceTask.getSlice() == null)");

            //check if it is a duplicate slice name error
            boolean isDuplicateSliceError = false;

            if(taskExecution.getException() instanceof JFedException){     //exception == null will return false
              JFedException ex = (JFedException)taskExecution.getException();

                if(ex.getXmlRpcReply() != null && ex.getXmlRpcReply().getOutput().endsWith(" already a registered slice")){
                    isDuplicateSliceError = true;
                    LOG.warn("Detected that exception of register experiment is a duplicate slice error");

                    updateMessage("Experiment name already in use. Restart the experiment with another name...");
                }else{
                    LOG.warn("Received an unrecognized other error: {}", ex.getXmlRpcReply() != null ? ex.getXmlRpcReply().getOutput() : "null");
                }
            }

            if(isDuplicateSliceError) {

                Platform.runLater(() -> JFDialogs.create()
                        .message("The experiment name that you requested is already in use on the slice authority. Please try again with another name.\n\n" +
                                "Check the Errors-tab for more information.")
                        .masthead("Experiment name already in use")
                        .title("Experiment registration failed")
                        .showError());
            }else {

                Platform.runLater(() -> JFDialogs.create()
                        .message("An error occurred when requesting the slice credential.\n\n." +
                                "Check the Errors-tab for more information.")
                        .masthead("Experiment registration failed")
                        .title("Experiment registration failed")
                        .showError());

            }
            //an error occurred, stop the process!
            experiment.setSliceState(SliceState.FAILED);
            return;
        }

        Platform.runLater(() -> {
            experiment.registerSlice(createSliceTask.getSlice());
            experiment.setSliceState(SliceState.CREATED);
        });
    }
}
