package be.iminds.ilabt.jfed.experimenter_gui.slice.tasks;

import be.iminds.ilabt.jfed.experimenter_gui.slice.Experiment;
import be.iminds.ilabt.jfed.experimenter_gui.ui.status.TaskStatusIndicator;
import be.iminds.ilabt.jfed.highlevel.controller.Task;
import be.iminds.ilabt.jfed.highlevel.controller.TaskExecution;

/**
 * Differs from SingleCallExperimentTask in that it expects the task to be already submitted to the taskthread!
 *
 * User: twalcari
 * Date: 2/2/2015
 * Time: 14:24
 */
public abstract class RunningCallExperimentTask extends ExperimentTask {
    private final Task task;
    private final TaskExecution taskExecution;

    RunningCallExperimentTask(Experiment experiment, Task task, TaskExecution taskExecution) {
        super(experiment);
        this.task = task;
        this.taskExecution = taskExecution;

    }

    @Override
    protected final TaskStatusIndicator.Status executeTask() {
        while (!taskExecution.isCompleted()) {
            try {
                Thread.sleep(250);
            } catch (InterruptedException ignored) {
            }
        }

        registerFinishedTaskExecution(task, taskExecution);
        onTaskFinished(taskExecution);

        switch (taskExecution.getState()) {
            case SUCCESS:
                return TaskStatusIndicator.Status.SUCCESS;
            case FAILED:
            case CANCELLED:
                return TaskStatusIndicator.Status.FAILED;
            default:
                throw new IllegalStateException("Did not expected state " + taskExecution.getState().name()
                        + " for finished task " + task.toString());
        }
    }

    public void onTaskFinished(TaskExecution taskExecution) {
    }

    public TaskExecution getTaskExecution() {
        return taskExecution;
    }

    public Task getTask() {
        return task;
    }

    @Override
    public String toString() {
        return getClass().getName() + "{" +
                "task=" + task.getName() +
                '}';
    }
}
