package be.iminds.ilabt.jfed.highlevel.model;

import javax.inject.Inject;
import javax.inject.Singleton;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

/**
 * EasyOcciModel stores OCCI specific data that does not belong in EasyModel
 *
 * //TODO make this a model with Observable properties
 */
@Singleton
public class EasyOcciModel {

    @Inject
    public EasyOcciModel() {
    }

    private final Map<String, Set<Network>> networksByLocationId = new HashMap<>();
    private final Map<String, Set<Storage>> storagesByLocationId = new HashMap<>();

    public static class Network {
        private final String id;
        private final String href;
        private final String name;
        private final String locationId;
        private final String locationHref;

        public Network(String id, String href, String name, String locationId, String locationHref) {
            this.id = id;
            this.href = href;
            this.name = name;
            this.locationId = locationId;
            this.locationHref = locationHref;
        }

        public String getHref() {
            return href; }
        public String getId() { return id; }
        public String getName() { return name; }
        public String getLocationId() { return locationId; }
        public String getLocationHref() { return locationHref; }

        @Override
        public boolean equals(Object o) {
            if (this == o) return true;
            if (o == null || getClass() != o.getClass()) return false;

            Network network = (Network) o;

            if (!id.equals(network.id)) return false;
            if (!locationId.equals(network.locationId)) return false;

            return true;
        }

        @Override
        public int hashCode() {
            int result = id.hashCode();
            result = 31 * result + locationId.hashCode();
            return result;
        }

        @Override public String toString() { return name; }
    }

    public static class Storage {
        private final String id;
        private final String href;
        private final String name;
        private final String locationId;
        private final String locationHref;

        public Storage(String id, String href, String name, String locationId, String locationHref) {
            this.id = id;
            this.href = href;
            this.name = name;
            this.locationId = locationId;
            this.locationHref = locationHref;
        }

        public String getHref() { return href; }
        public String getId() { return id; }
        public String getName() { return name; }
        public String getLocationId() { return locationId; }
        public String getLocationHref() { return locationHref; }

        @Override
        public boolean equals(Object o) {
            if (this == o) return true;
            if (o == null || getClass() != o.getClass()) return false;

            Storage storage = (Storage) o;

            if (!id.equals(storage.id)) return false;
            if (!locationId.equals(storage.locationId)) return false;

            return true;
        }

        @Override
        public int hashCode() {
            int result = id.hashCode();
            result = 31 * result + locationId.hashCode();
            return result;
        }

        @Override public String toString() { return name; }
    }

    public Set<Network> getNetworks(String locationId) {
        return networksByLocationId.get(locationId);
    }

    public Set<Storage> getStorages(String locationId) {
        return storagesByLocationId.get(locationId);
    }

    public void noteLocationExists(String locationId, String locationHref) {
        //not interested
    }

    public void noteNetworkExists(String locationId, String locationHref, String name, String networkId, String networkHref) {
        Set<Network> networks = networksByLocationId.get(locationId);
        if (networks == null) {
            networks = new HashSet<>();
            networksByLocationId.put(locationId, networks);
        }
        networks.add(new Network(networkId, networkHref, name, locationId, locationHref));
    }

    public void noteStorageExists(String locationId, String locationHref, String name, String storageId, String storageHref) {
        Set<Storage> storages = storagesByLocationId.get(locationId);
        if (storages == null) {
            storages = new HashSet<>();
            storagesByLocationId.put(locationId, storages);
        }
        storages.add(new Storage(storageId, storageHref, name, locationId, locationHref));
    }
}
