package be.iminds.ilabt.jfed.lowlevel.authority;

import be.iminds.ilabt.jfed.preferences.CorePreferenceKey;
import be.iminds.ilabt.jfed.preferences.JFedCorePreferences;
import be.iminds.ilabt.jfed.preferences.JFedPreferences;
import be.iminds.ilabt.jfed.util.GeniUrn;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.DataProvider;
import org.testng.annotations.Test;

import java.util.ArrayList;
import java.util.List;

import static org.hamcrest.MatcherAssert.assertThat;
import static org.hamcrest.Matchers.*;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

import static org.testng.Assert.*;

public class AuthorityListModelTest {
    private final static String ONLY_TOP_EXISTS_TOP_URN_STRING = "urn:publicid:IDN+wall2.ilabt.iminds.be+authority+cm";
    private final static String ONLY_TOP_EXISTS_SUB_URN_STRING = "urn:publicid:IDN+wall2.ilabt.iminds.be:fakesubauth+authority+cm";
    private final static String ONLY_TOP_EXISTS_SUB_SUB_SUB_URN_STRING = "urn:publicid:IDN+wall2.ilabt.iminds.be:fakesubauth:fake2:fake3+authority+cm";
    private final static String ONLY_SUB_EXISTS_TOP_URN_STRING = "urn:publicid:IDN+ple+authority+cm";
    private final static String ONLY_SUB_EXISTS_SUB_URN_STRING = "urn:publicid:IDN+ple:ibbtple+authority+cm";
    private final static String ONLY_SUB_EXISTS_SUB_SUB_SUB_SUB_URN_STRING = "urn:publicid:IDN+ple:ibbtple:fakesub:fakesub2:fakesub3+authority+cm";
    private final static String EXOGENI_URN_STRING = "urn:publicid:IDN+exogeni.net+authority+am";
    private final static String EXOGENI_BBN_URN_STRING = "urn:publicid:IDN+exogeni.net:bbnvmsite+authority+am";
    private final static String EXOGENI_BBN_FAKESUB_URN_STRING = "urn:publicid:IDN+exogeni.net:bbnvmsite:fakesub+authority+am";

    private final static GeniUrn ONLY_TOP_EXISTS_TOP_URN = GeniUrn.parse(ONLY_TOP_EXISTS_TOP_URN_STRING);
    private final static GeniUrn ONLY_TOP_EXISTS_SUB_URN = GeniUrn.parse(ONLY_TOP_EXISTS_SUB_URN_STRING);
    private final static GeniUrn ONLY_TOP_EXISTS_SUB_SUB_SUB_URN = GeniUrn.parse(ONLY_TOP_EXISTS_SUB_SUB_SUB_URN_STRING);
    private final static GeniUrn ONLY_SUB_EXISTS_TOP_URN = GeniUrn.parse(ONLY_SUB_EXISTS_TOP_URN_STRING);
    private final static GeniUrn ONLY_SUB_EXISTS_SUB_URN = GeniUrn.parse(ONLY_SUB_EXISTS_SUB_URN_STRING);
    private final static GeniUrn ONLY_SUB_EXISTS_SUB_SUB_SUB_SUB_URN = GeniUrn.parse(ONLY_SUB_EXISTS_SUB_SUB_SUB_SUB_URN_STRING);
    private final static GeniUrn EXOGENI_URN = GeniUrn.parse(EXOGENI_URN_STRING);
    private final static GeniUrn EXOGENI_BBN_URN = GeniUrn.parse(EXOGENI_BBN_URN_STRING);
    private final static GeniUrn EXOGENI_BBN_FAKESUB_URN = GeniUrn.parse(EXOGENI_BBN_FAKESUB_URN_STRING);

    private JFedPreferences jFedPreferences;
    private AuthorityListModel authorityListModel;
    private AuthorityFinder authorityFinder;

    private boolean exoPrefersTopLevel = true;

    @BeforeClass
    public void setup() {
        jFedPreferences = new JFedCorePreferences(); //note: corePreferences does NOT autosave
        jFedPreferences.setString(CorePreferenceKey.PREF_TESTBEDS_XML_SOURCE, "BUILTIN");
        authorityListModel = new JFedAuthorityList(null/*do not use default authorities file*/, jFedPreferences).get();
        authorityFinder = new AuthorityFinder(authorityListModel, jFedPreferences);

        exoPrefersTopLevel = authorityFinder.findByUrn(EXOGENI_URN, AuthorityFinder.Purpose.RECOVER).hasSpecialCase(SfaAuthority.SpecialCase.PREFER_TOPLEVEL);
    }


    @DataProvider(name = "ignoreUrnTypeAndName")
    public static Object[][] ignoreUrnTypeAndName() {

        Object[][] res = new Object[2][];
        res[0] = new Object[] { true };
        res[1] = new Object[] { false };
        return res;
    }

    @Test(dataProvider = "ignoreUrnTypeAndName")
    public void testGetByUrnDirect1(boolean ignoreUrnTypeAndName) {
        AuthorityListModel.SubAuthMatchAllowed allowed = AuthorityListModel.SubAuthMatchAllowed.ALLOW_ONLY_EXACT_SUBAUTHORITY;
        AuthorityListModel.SubAuthMatchPreference preferred = AuthorityListModel.SubAuthMatchPreference.PREFER_EXACT_SUBAUTHORITY;

        assertThat(authorityListModel.getByUrn(EXOGENI_URN, allowed, preferred, ignoreUrnTypeAndName).getUrn(), equalTo(EXOGENI_URN));
        assertThat(authorityListModel.getByUrn(EXOGENI_BBN_URN, allowed, preferred, ignoreUrnTypeAndName).getUrn(), equalTo(EXOGENI_BBN_URN));
        assertThat(authorityListModel.getByUrn(EXOGENI_BBN_FAKESUB_URN, allowed, preferred, ignoreUrnTypeAndName), is(nullValue()));
        assertThat(authorityListModel.getByUrn(ONLY_TOP_EXISTS_TOP_URN, allowed, preferred, ignoreUrnTypeAndName).getUrn(), equalTo(ONLY_TOP_EXISTS_TOP_URN));
        assertThat(authorityListModel.getByUrn(ONLY_TOP_EXISTS_SUB_URN, allowed, preferred, ignoreUrnTypeAndName), is(nullValue()));
        assertThat(authorityListModel.getByUrn(ONLY_TOP_EXISTS_SUB_SUB_SUB_URN, allowed, preferred, ignoreUrnTypeAndName), is(nullValue()));
        assertThat(authorityListModel.getByUrn(ONLY_SUB_EXISTS_SUB_URN, allowed, preferred, ignoreUrnTypeAndName).getUrn(), equalTo(ONLY_SUB_EXISTS_SUB_URN));
        assertThat(authorityListModel.getByUrn(ONLY_SUB_EXISTS_SUB_SUB_SUB_SUB_URN, allowed, preferred, ignoreUrnTypeAndName), is(nullValue()));
        assertThat(authorityListModel.getByUrn(ONLY_SUB_EXISTS_TOP_URN, allowed, preferred, ignoreUrnTypeAndName), is(nullValue()));
    }
    @Test(dataProvider = "ignoreUrnTypeAndName")
    public void testGetByUrnDirect2(boolean ignoreUrnTypeAndName) {
        AuthorityListModel.SubAuthMatchAllowed allowed = AuthorityListModel.SubAuthMatchAllowed.ALLOW_TOPLEVEL;
        AuthorityListModel.SubAuthMatchPreference preferred = AuthorityListModel.SubAuthMatchPreference.PREFER_EXACT_SUBAUTHORITY;

        assertThat(authorityListModel.getByUrn(EXOGENI_URN, allowed, preferred, ignoreUrnTypeAndName).getUrn(), equalTo(EXOGENI_URN));
        assertThat(authorityListModel.getByUrn(EXOGENI_BBN_URN, allowed, preferred, ignoreUrnTypeAndName).getUrn(), equalTo(EXOGENI_BBN_URN));
        assertThat(authorityListModel.getByUrn(EXOGENI_BBN_FAKESUB_URN, allowed, preferred, ignoreUrnTypeAndName).getUrn(), equalTo(EXOGENI_BBN_URN));
        assertThat(authorityListModel.getByUrn(ONLY_TOP_EXISTS_TOP_URN, allowed, preferred, ignoreUrnTypeAndName).getUrn(), equalTo(ONLY_TOP_EXISTS_TOP_URN));
        assertThat(authorityListModel.getByUrn(ONLY_TOP_EXISTS_SUB_URN, allowed, preferred, ignoreUrnTypeAndName).getUrn(), equalTo(ONLY_TOP_EXISTS_TOP_URN));
        assertThat(authorityListModel.getByUrn(ONLY_TOP_EXISTS_SUB_SUB_SUB_URN, allowed, preferred, ignoreUrnTypeAndName).getUrn(), equalTo(ONLY_TOP_EXISTS_TOP_URN));
        assertThat(authorityListModel.getByUrn(ONLY_SUB_EXISTS_SUB_URN, allowed, preferred, ignoreUrnTypeAndName).getUrn(), equalTo(ONLY_SUB_EXISTS_SUB_URN));
        assertThat(authorityListModel.getByUrn(ONLY_SUB_EXISTS_SUB_SUB_SUB_SUB_URN, allowed, preferred, ignoreUrnTypeAndName).getUrn(), equalTo(ONLY_SUB_EXISTS_SUB_URN));
        assertThat(authorityListModel.getByUrn(ONLY_SUB_EXISTS_TOP_URN, allowed, preferred, ignoreUrnTypeAndName), is(nullValue()));
    }
    @Test(dataProvider = "ignoreUrnTypeAndName")
    public void testGetByUrnDirect3(boolean ignoreUrnTypeAndName) {
        AuthorityListModel.SubAuthMatchAllowed allowed = AuthorityListModel.SubAuthMatchAllowed.ALLOW_OTHER_SUBAUTHORITY;
        AuthorityListModel.SubAuthMatchPreference preferred = AuthorityListModel.SubAuthMatchPreference.PREFER_EXACT_SUBAUTHORITY;

        assertThat(authorityListModel.getByUrn(EXOGENI_URN, allowed, preferred, ignoreUrnTypeAndName).getUrn(), equalTo(EXOGENI_URN));
        assertThat(authorityListModel.getByUrn(EXOGENI_BBN_URN, allowed, preferred, ignoreUrnTypeAndName).getUrn(), equalTo(EXOGENI_BBN_URN));
        assertThat(authorityListModel.getByUrn(EXOGENI_BBN_FAKESUB_URN, allowed, preferred, ignoreUrnTypeAndName).getUrn(), equalTo(EXOGENI_BBN_URN));
        assertThat(authorityListModel.getByUrn(ONLY_TOP_EXISTS_TOP_URN, allowed, preferred, ignoreUrnTypeAndName).getUrn(), equalTo(ONLY_TOP_EXISTS_TOP_URN));
        assertThat(authorityListModel.getByUrn(ONLY_TOP_EXISTS_SUB_URN, allowed, preferred, ignoreUrnTypeAndName).getUrn(), equalTo(ONLY_TOP_EXISTS_TOP_URN));
        assertThat(authorityListModel.getByUrn(ONLY_TOP_EXISTS_SUB_SUB_SUB_URN, allowed, preferred, ignoreUrnTypeAndName).getUrn(), equalTo(ONLY_TOP_EXISTS_TOP_URN));
        assertThat(authorityListModel.getByUrn(ONLY_SUB_EXISTS_SUB_URN, allowed, preferred, ignoreUrnTypeAndName).getUrn(), equalTo(ONLY_SUB_EXISTS_SUB_URN));
        assertThat(authorityListModel.getByUrn(ONLY_SUB_EXISTS_SUB_SUB_SUB_SUB_URN, allowed, preferred, ignoreUrnTypeAndName).getUrn(), equalTo(ONLY_SUB_EXISTS_SUB_URN));
        assertThat(authorityListModel.getByUrn(ONLY_SUB_EXISTS_TOP_URN, allowed, preferred, ignoreUrnTypeAndName).getUrn(), equalTo(ONLY_SUB_EXISTS_SUB_URN));
    }
    @Test(dataProvider = "ignoreUrnTypeAndName")
    public void testGetByUrnDirect4(boolean ignoreUrnTypeAndName) {
        AuthorityListModel.SubAuthMatchAllowed allowed = AuthorityListModel.SubAuthMatchAllowed.ALLOW_ONLY_EXACT_SUBAUTHORITY;
        AuthorityListModel.SubAuthMatchPreference preferred = AuthorityListModel.SubAuthMatchPreference.PREFER_TOPLEVEL;

        assertThat(authorityListModel.getByUrn(EXOGENI_URN, allowed, preferred, ignoreUrnTypeAndName).getUrn(), equalTo(EXOGENI_URN));
        assertThat(authorityListModel.getByUrn(EXOGENI_BBN_URN, allowed, preferred, ignoreUrnTypeAndName).getUrn(), equalTo(EXOGENI_BBN_URN));
        assertThat(authorityListModel.getByUrn(EXOGENI_BBN_FAKESUB_URN, allowed, preferred, ignoreUrnTypeAndName), is(nullValue()));
        assertThat(authorityListModel.getByUrn(ONLY_TOP_EXISTS_TOP_URN, allowed, preferred, ignoreUrnTypeAndName).getUrn(), equalTo(ONLY_TOP_EXISTS_TOP_URN));
        assertThat(authorityListModel.getByUrn(ONLY_TOP_EXISTS_SUB_URN, allowed, preferred, ignoreUrnTypeAndName), is(nullValue()));
        assertThat(authorityListModel.getByUrn(ONLY_TOP_EXISTS_SUB_SUB_SUB_URN, allowed, preferred, ignoreUrnTypeAndName), is(nullValue()));
        assertThat(authorityListModel.getByUrn(ONLY_SUB_EXISTS_SUB_URN, allowed, preferred, ignoreUrnTypeAndName).getUrn(), equalTo(ONLY_SUB_EXISTS_SUB_URN));
        assertThat(authorityListModel.getByUrn(ONLY_SUB_EXISTS_SUB_SUB_SUB_SUB_URN, allowed, preferred, ignoreUrnTypeAndName), is(nullValue()));
        assertThat(authorityListModel.getByUrn(ONLY_SUB_EXISTS_TOP_URN, allowed, preferred, ignoreUrnTypeAndName), is(nullValue()));
    }
    @Test(dataProvider = "ignoreUrnTypeAndName")
    public void testGetByUrnDirect5(boolean ignoreUrnTypeAndName) {
        AuthorityListModel.SubAuthMatchAllowed allowed = AuthorityListModel.SubAuthMatchAllowed.ALLOW_TOPLEVEL;
        AuthorityListModel.SubAuthMatchPreference preferred = AuthorityListModel.SubAuthMatchPreference.PREFER_TOPLEVEL;

        assertThat(authorityListModel.getByUrn(EXOGENI_URN, allowed, preferred, ignoreUrnTypeAndName).getUrn(), equalTo(EXOGENI_URN));
        assertThat(authorityListModel.getByUrn(EXOGENI_BBN_URN, allowed, preferred, ignoreUrnTypeAndName).getUrn(), equalTo(EXOGENI_URN));
        assertThat(authorityListModel.getByUrn(EXOGENI_BBN_FAKESUB_URN, allowed, preferred, ignoreUrnTypeAndName).getUrn(), equalTo(EXOGENI_URN));
        assertThat(authorityListModel.getByUrn(ONLY_TOP_EXISTS_TOP_URN, allowed, preferred, ignoreUrnTypeAndName).getUrn(), equalTo(ONLY_TOP_EXISTS_TOP_URN));
        assertThat(authorityListModel.getByUrn(ONLY_TOP_EXISTS_SUB_URN, allowed, preferred, ignoreUrnTypeAndName).getUrn(), equalTo(ONLY_TOP_EXISTS_TOP_URN));
        assertThat(authorityListModel.getByUrn(ONLY_TOP_EXISTS_SUB_SUB_SUB_URN, allowed, preferred, ignoreUrnTypeAndName).getUrn(), equalTo(ONLY_TOP_EXISTS_TOP_URN));
        assertThat(authorityListModel.getByUrn(ONLY_SUB_EXISTS_SUB_URN, allowed, preferred, ignoreUrnTypeAndName).getUrn(), equalTo(ONLY_SUB_EXISTS_SUB_URN));
        assertThat(authorityListModel.getByUrn(ONLY_SUB_EXISTS_SUB_SUB_SUB_SUB_URN, allowed, preferred, ignoreUrnTypeAndName).getUrn(), equalTo(ONLY_SUB_EXISTS_SUB_URN));
        assertThat(authorityListModel.getByUrn(ONLY_SUB_EXISTS_TOP_URN, allowed, preferred, ignoreUrnTypeAndName), is(nullValue()));
    }
    @Test(dataProvider = "ignoreUrnTypeAndName")
    public void testGetByUrnDirect6(boolean ignoreUrnTypeAndName) {
        AuthorityListModel.SubAuthMatchAllowed allowed = AuthorityListModel.SubAuthMatchAllowed.ALLOW_OTHER_SUBAUTHORITY;
        AuthorityListModel.SubAuthMatchPreference preferred = AuthorityListModel.SubAuthMatchPreference.PREFER_TOPLEVEL;

        assertThat(authorityListModel.getByUrn(EXOGENI_URN, allowed, preferred, ignoreUrnTypeAndName).getUrn(), equalTo(EXOGENI_URN));
        assertThat(authorityListModel.getByUrn(EXOGENI_BBN_URN, allowed, preferred, ignoreUrnTypeAndName).getUrn(), equalTo(EXOGENI_URN));
        assertThat(authorityListModel.getByUrn(EXOGENI_BBN_FAKESUB_URN, allowed, preferred, ignoreUrnTypeAndName).getUrn(), equalTo(EXOGENI_URN));
        assertThat(authorityListModel.getByUrn(ONLY_TOP_EXISTS_TOP_URN, allowed, preferred, ignoreUrnTypeAndName).getUrn(), equalTo(ONLY_TOP_EXISTS_TOP_URN));
        assertThat(authorityListModel.getByUrn(ONLY_TOP_EXISTS_SUB_URN, allowed, preferred, ignoreUrnTypeAndName).getUrn(), equalTo(ONLY_TOP_EXISTS_TOP_URN));
        assertThat(authorityListModel.getByUrn(ONLY_TOP_EXISTS_SUB_SUB_SUB_URN, allowed, preferred, ignoreUrnTypeAndName).getUrn(), equalTo(ONLY_TOP_EXISTS_TOP_URN));
        assertThat(authorityListModel.getByUrn(ONLY_SUB_EXISTS_SUB_URN, allowed, preferred, ignoreUrnTypeAndName).getUrn(), equalTo(ONLY_SUB_EXISTS_SUB_URN));
        assertThat(authorityListModel.getByUrn(ONLY_SUB_EXISTS_SUB_SUB_SUB_SUB_URN, allowed, preferred, ignoreUrnTypeAndName).getUrn(), equalTo(ONLY_SUB_EXISTS_SUB_URN));
        assertThat(authorityListModel.getByUrn(ONLY_SUB_EXISTS_TOP_URN, allowed, preferred, ignoreUrnTypeAndName).getUrn(), equalTo(ONLY_SUB_EXISTS_SUB_URN));
    }


    @DataProvider(name = "forceExoSmConfig")
    public static Object[][] forceExoSmConfig() {

        Object[][] res = new Object[3][];
        res[0] = new Object[] { "YES" };
        res[1] = new Object[] { "NO" };
        res[2] = new Object[] { "AUTO" };
        return res;
    }


    @Test(dataProvider = "forceExoSmConfig")
    public void testGetByUrnPurpose1(String exoSmConfig) {
        AuthorityFinder.Purpose purpose = AuthorityFinder.Purpose.CREATE_SLIVER;
        jFedPreferences.setString(CorePreferenceKey.PREF_TESTBEDSPECIFIC_EXOGENI_FORCE_EXOSM, exoSmConfig);
        assertThat(authorityFinder.findByUrn(EXOGENI_URN, purpose).getUrn(), equalTo(EXOGENI_URN));

        if (exoSmConfig.equalsIgnoreCase("yes") || (exoSmConfig.equalsIgnoreCase("auto") && exoPrefersTopLevel)) {
            assertThat(authorityFinder.findByUrn(EXOGENI_BBN_URN, purpose).getUrn(), equalTo(EXOGENI_URN));
            assertThat(authorityFinder.findByUrn(EXOGENI_BBN_FAKESUB_URN, purpose).getUrn(), equalTo(EXOGENI_URN));
        }
        if (exoSmConfig.equalsIgnoreCase("no") || (exoSmConfig.equalsIgnoreCase("auto") && !exoPrefersTopLevel)) {
            assertThat(authorityFinder.findByUrn(EXOGENI_BBN_URN, purpose).getUrn(), equalTo(EXOGENI_BBN_URN));
            assertThat(authorityFinder.findByUrn(EXOGENI_BBN_FAKESUB_URN, purpose).getUrn(), equalTo(EXOGENI_BBN_URN));
        }

        assertThat(authorityFinder.findByUrn(ONLY_TOP_EXISTS_TOP_URN, purpose).getUrn(), equalTo(ONLY_TOP_EXISTS_TOP_URN));
        assertThat(authorityFinder.findByUrn(ONLY_TOP_EXISTS_SUB_URN, purpose).getUrn(), equalTo(ONLY_TOP_EXISTS_TOP_URN));
        assertThat(authorityFinder.findByUrn(ONLY_TOP_EXISTS_SUB_SUB_SUB_URN, purpose).getUrn(), equalTo(ONLY_TOP_EXISTS_TOP_URN));
        assertThat(authorityFinder.findByUrn(ONLY_SUB_EXISTS_SUB_URN, purpose).getUrn(), equalTo(ONLY_SUB_EXISTS_SUB_URN));
        assertThat(authorityFinder.findByUrn(ONLY_SUB_EXISTS_SUB_SUB_SUB_SUB_URN, purpose).getUrn(), equalTo(ONLY_SUB_EXISTS_SUB_URN));
//        assertThat(authorityFinder.findByUrn(ONLY_SUB_EXISTS_TOP_URN, purpose).getUrn(), equalTo(ONLY_SUB_EXISTS_SUB_URN));
        assertThat(authorityFinder.findByUrn(ONLY_SUB_EXISTS_TOP_URN, purpose), is(nullValue()));
    }
    @Test(dataProvider = "forceExoSmConfig")
    public void testGetByUrnPurpose2(String exoSmConfig) {
        AuthorityFinder.Purpose purpose = AuthorityFinder.Purpose.RECOVER;

        jFedPreferences.setString(CorePreferenceKey.PREF_TESTBEDSPECIFIC_EXOGENI_FORCE_EXOSM, exoSmConfig);
        assertThat(authorityFinder.findByUrn(EXOGENI_URN, purpose).getUrn(), equalTo(EXOGENI_URN));

        if (exoSmConfig.equalsIgnoreCase("yes") || (exoSmConfig.equalsIgnoreCase("auto") && exoPrefersTopLevel)) {
            assertThat(authorityFinder.findByUrn(EXOGENI_BBN_URN, purpose).getUrn(), equalTo(EXOGENI_URN));
            assertThat(authorityFinder.findByUrn(EXOGENI_BBN_FAKESUB_URN, purpose).getUrn(), equalTo(EXOGENI_URN));
        }
        if (exoSmConfig.equalsIgnoreCase("no") || (exoSmConfig.equalsIgnoreCase("auto") && !exoPrefersTopLevel)) {
            assertThat(authorityFinder.findByUrn(EXOGENI_BBN_URN, purpose).getUrn(), equalTo(EXOGENI_BBN_URN));
            assertThat(authorityFinder.findByUrn(EXOGENI_BBN_FAKESUB_URN, purpose).getUrn(), equalTo(EXOGENI_BBN_URN));
        }

        assertThat(authorityFinder.findByUrn(ONLY_TOP_EXISTS_TOP_URN, purpose).getUrn(), equalTo(ONLY_TOP_EXISTS_TOP_URN));
        assertThat(authorityFinder.findByUrn(ONLY_TOP_EXISTS_SUB_URN, purpose).getUrn(), equalTo(ONLY_TOP_EXISTS_TOP_URN));
        assertThat(authorityFinder.findByUrn(ONLY_TOP_EXISTS_SUB_SUB_SUB_URN, purpose).getUrn(), equalTo(ONLY_TOP_EXISTS_TOP_URN));
        assertThat(authorityFinder.findByUrn(ONLY_SUB_EXISTS_SUB_URN, purpose).getUrn(), equalTo(ONLY_SUB_EXISTS_SUB_URN));
        assertThat(authorityFinder.findByUrn(ONLY_SUB_EXISTS_SUB_SUB_SUB_SUB_URN, purpose).getUrn(), equalTo(ONLY_SUB_EXISTS_SUB_URN));
        assertThat(authorityFinder.findByUrn(ONLY_SUB_EXISTS_TOP_URN, purpose), is(nullValue()));
    }
    @Test
    public void testGetByUrnPurpose3() {
        AuthorityFinder.Purpose purpose = AuthorityFinder.Purpose.RECOVER_BOTH_FIRST;

        assertThat(authorityFinder.findByUrn(EXOGENI_URN, purpose).getUrn(), equalTo(EXOGENI_URN));
        assertThat(authorityFinder.findByUrn(EXOGENI_BBN_URN, purpose).getUrn(), equalTo(EXOGENI_URN));
        assertThat(authorityFinder.findByUrn(EXOGENI_BBN_FAKESUB_URN, purpose).getUrn(), equalTo(EXOGENI_URN));
        assertThat(authorityFinder.findByUrn(ONLY_TOP_EXISTS_TOP_URN, purpose).getUrn(), equalTo(ONLY_TOP_EXISTS_TOP_URN));
        assertThat(authorityFinder.findByUrn(ONLY_TOP_EXISTS_SUB_URN, purpose).getUrn(), equalTo(ONLY_TOP_EXISTS_TOP_URN));
        assertThat(authorityFinder.findByUrn(ONLY_TOP_EXISTS_SUB_SUB_SUB_URN, purpose).getUrn(), equalTo(ONLY_TOP_EXISTS_TOP_URN));
        assertThat(authorityFinder.findByUrn(ONLY_SUB_EXISTS_SUB_URN, purpose).getUrn(), equalTo(ONLY_SUB_EXISTS_SUB_URN));
        assertThat(authorityFinder.findByUrn(ONLY_SUB_EXISTS_SUB_SUB_SUB_SUB_URN, purpose).getUrn(), equalTo(ONLY_SUB_EXISTS_SUB_URN));
        assertThat(authorityFinder.findByUrn(ONLY_SUB_EXISTS_TOP_URN, purpose), is(nullValue()));
    }
    @Test
    public void testGetByUrnPurpose4() {
        AuthorityFinder.Purpose purpose = AuthorityFinder.Purpose.RECOVER_BOTH_SECOND;

        assertThat(authorityFinder.findByUrn(EXOGENI_URN, purpose).getUrn(), equalTo(EXOGENI_URN));
        assertThat(authorityFinder.findByUrn(EXOGENI_BBN_URN, purpose).getUrn(), equalTo(EXOGENI_BBN_URN));
        assertThat(authorityFinder.findByUrn(EXOGENI_BBN_FAKESUB_URN, purpose).getUrn(), equalTo(EXOGENI_BBN_URN));
        assertThat(authorityFinder.findByUrn(ONLY_TOP_EXISTS_TOP_URN, purpose).getUrn(), equalTo(ONLY_TOP_EXISTS_TOP_URN));
        assertThat(authorityFinder.findByUrn(ONLY_TOP_EXISTS_SUB_URN, purpose).getUrn(), equalTo(ONLY_TOP_EXISTS_TOP_URN));
        assertThat(authorityFinder.findByUrn(ONLY_TOP_EXISTS_SUB_SUB_SUB_URN, purpose).getUrn(), equalTo(ONLY_TOP_EXISTS_TOP_URN));
        assertThat(authorityFinder.findByUrn(ONLY_SUB_EXISTS_SUB_URN, purpose).getUrn(), equalTo(ONLY_SUB_EXISTS_SUB_URN));
        assertThat(authorityFinder.findByUrn(ONLY_SUB_EXISTS_SUB_SUB_SUB_SUB_URN, purpose).getUrn(), equalTo(ONLY_SUB_EXISTS_SUB_URN));
        assertThat(authorityFinder.findByUrn(ONLY_SUB_EXISTS_TOP_URN, purpose), is(nullValue()));
    }
    @Test
    public void testGetByUrnPurpose5() {
        AuthorityFinder.Purpose purpose = AuthorityFinder.Purpose.REQUEST_RSPEC;

        assertThat(authorityFinder.findByUrn(EXOGENI_URN, purpose).getUrn(), equalTo(EXOGENI_URN));
        assertThat(authorityFinder.findByUrn(EXOGENI_BBN_URN, purpose).getUrn(), equalTo(EXOGENI_BBN_URN));
        assertThat(authorityFinder.findByUrn(EXOGENI_BBN_FAKESUB_URN, purpose), is(nullValue()));
        assertThat(authorityFinder.findByUrn(ONLY_TOP_EXISTS_TOP_URN, purpose).getUrn(), equalTo(ONLY_TOP_EXISTS_TOP_URN));
        assertThat(authorityFinder.findByUrn(ONLY_TOP_EXISTS_SUB_URN, purpose), is(nullValue()));
        assertThat(authorityFinder.findByUrn(ONLY_TOP_EXISTS_SUB_SUB_SUB_URN, purpose), is(nullValue()));
        assertThat(authorityFinder.findByUrn(ONLY_SUB_EXISTS_SUB_URN, purpose).getUrn(), equalTo(ONLY_SUB_EXISTS_SUB_URN));
        assertThat(authorityFinder.findByUrn(ONLY_SUB_EXISTS_SUB_SUB_SUB_SUB_URN, purpose), is(nullValue()));
        assertThat(authorityFinder.findByUrn(ONLY_SUB_EXISTS_TOP_URN, purpose), is(nullValue()));
    }
    @Test(dataProvider = "forceExoSmConfig")
    public void testGetByUrnPurpose6(String exoSmConfig) {
        AuthorityFinder.Purpose purpose = AuthorityFinder.Purpose.STATUS;
        jFedPreferences.setString(CorePreferenceKey.PREF_TESTBEDSPECIFIC_EXOGENI_FORCE_EXOSM, exoSmConfig);

        assertThat(authorityFinder.findByUrn(EXOGENI_URN, purpose).getUrn(), equalTo(EXOGENI_URN));

        if (exoSmConfig.equalsIgnoreCase("yes") || (exoSmConfig.equalsIgnoreCase("auto") && exoPrefersTopLevel)) {
            assertThat(authorityFinder.findByUrn(EXOGENI_BBN_URN, purpose).getUrn(), equalTo(EXOGENI_URN));
            assertThat(authorityFinder.findByUrn(EXOGENI_BBN_FAKESUB_URN, purpose).getUrn(), equalTo(EXOGENI_URN));
        }
        if (exoSmConfig.equalsIgnoreCase("no") || (exoSmConfig.equalsIgnoreCase("auto") && !exoPrefersTopLevel)) {
            assertThat(authorityFinder.findByUrn(EXOGENI_BBN_URN, purpose).getUrn(), equalTo(EXOGENI_BBN_URN));
            assertThat(authorityFinder.findByUrn(EXOGENI_BBN_FAKESUB_URN, purpose).getUrn(), equalTo(EXOGENI_BBN_URN));
        }

        assertThat(authorityFinder.findByUrn(ONLY_TOP_EXISTS_TOP_URN, purpose).getUrn(), equalTo(ONLY_TOP_EXISTS_TOP_URN));
        assertThat(authorityFinder.findByUrn(ONLY_TOP_EXISTS_SUB_URN, purpose).getUrn(), equalTo(ONLY_TOP_EXISTS_TOP_URN));
        assertThat(authorityFinder.findByUrn(ONLY_TOP_EXISTS_SUB_SUB_SUB_URN, purpose).getUrn(), equalTo(ONLY_TOP_EXISTS_TOP_URN));
        assertThat(authorityFinder.findByUrn(ONLY_SUB_EXISTS_SUB_URN, purpose).getUrn(), equalTo(ONLY_SUB_EXISTS_SUB_URN));
        assertThat(authorityFinder.findByUrn(ONLY_SUB_EXISTS_SUB_SUB_SUB_SUB_URN, purpose).getUrn(), equalTo(ONLY_SUB_EXISTS_SUB_URN));
        assertThat(authorityFinder.findByUrn(ONLY_SUB_EXISTS_TOP_URN, purpose), is(nullValue()));
    }
    @Test(dataProvider = "forceExoSmConfig")
    public void testGetByUrnPurpose7(String exoSmConfig) {
        AuthorityFinder.Purpose purpose = AuthorityFinder.Purpose.STITCHING;
        jFedPreferences.setString(CorePreferenceKey.PREF_TESTBEDSPECIFIC_EXOGENI_FORCE_EXOSM, exoSmConfig);

        assertThat(authorityFinder.findByUrn(EXOGENI_URN, purpose).getUrn(), equalTo(EXOGENI_URN));
        assertThat(authorityFinder.findByUrn(EXOGENI_BBN_URN, purpose).getUrn(), equalTo(EXOGENI_BBN_URN));
        assertThat(authorityFinder.findByUrn(EXOGENI_BBN_FAKESUB_URN, purpose), is(nullValue()));
        assertThat(authorityFinder.findByUrn(ONLY_TOP_EXISTS_TOP_URN, purpose).getUrn(), equalTo(ONLY_TOP_EXISTS_TOP_URN));
        assertThat(authorityFinder.findByUrn(ONLY_TOP_EXISTS_SUB_URN, purpose), is(nullValue()));
        assertThat(authorityFinder.findByUrn(ONLY_TOP_EXISTS_SUB_SUB_SUB_URN, purpose), is(nullValue()));
        assertThat(authorityFinder.findByUrn(ONLY_SUB_EXISTS_SUB_URN, purpose).getUrn(), equalTo(ONLY_SUB_EXISTS_SUB_URN));
        assertThat(authorityFinder.findByUrn(ONLY_SUB_EXISTS_SUB_SUB_SUB_SUB_URN, purpose), is(nullValue()));
        assertThat(authorityFinder.findByUrn(ONLY_SUB_EXISTS_TOP_URN, purpose), is(nullValue()));
    }

    @Test
    public void testSortBySubAuthDifference() throws Exception {
        List<SfaAuthority> start = new ArrayList<SfaAuthority>();
        start.add(new SfaAuthority("urn:publicid:IDN+example.net+authority+cm"));
        start.add(new SfaAuthority("urn:publicid:IDN+example.com:one+authority+cm"));
        start.add(new SfaAuthority("urn:publicid:IDN+ex.com:xxxxxxxxxxxxxxxxxxxxxxx:2+authority+cm"));
        start.add(new SfaAuthority("urn:publicid:IDN+example.com:aaaaa:bbbbb:three+authority+cm"));
        start.add(new SfaAuthority("urn:publicid:IDN+example.com:c:d:e:four+authority+cm"));

        List<SfaAuthority> actual = AuthorityListModel.sortBySubAuthDifference(start, new GeniUrn("urn:publicid:IDN+example.com:u:two+authority+cm"));
        System.out.println("actual=" + actual);

        assertThat(actual, is(start));
        assertThat(actual, hasSize(5));
        assertThat(actual.get(0).getUrn(), equalTo(new GeniUrn("urn:publicid:IDN+ex.com:xxxxxxxxxxxxxxxxxxxxxxx:2+authority+cm")));
        assertThat(actual.get(1).getUrn(), isOneOf(new GeniUrn("urn:publicid:IDN+example.com:one+authority+cm"), new GeniUrn("urn:publicid:IDN+example.com:aaaaa:bbbbb:three+authority+cm")));
        assertThat(actual.get(2).getUrn(), isOneOf(new GeniUrn("urn:publicid:IDN+example.com:one+authority+cm"), new GeniUrn("urn:publicid:IDN+example.com:aaaaa:bbbbb:three+authority+cm")));
        assertThat(actual.get(1).getUrn(), not(equalTo(actual.get(2).getUrn())));
        assertThat(actual.get(3).getUrn(), isOneOf(new GeniUrn("urn:publicid:IDN+example.net+authority+cm"), new GeniUrn("urn:publicid:IDN+example.com:c:d:e:four+authority+cm")));
        assertThat(actual.get(4).getUrn(), isOneOf(new GeniUrn("urn:publicid:IDN+example.net+authority+cm"), new GeniUrn("urn:publicid:IDN+example.com:c:d:e:four+authority+cm")));
        assertThat(actual.get(3).getUrn(), not(equalTo(actual.get(4).getUrn())));
    }

    @Test
    public void testSortBySubAuthNumber() throws Exception {
        List<SfaAuthority> start = new ArrayList<SfaAuthority>();
        start.add(new SfaAuthority("urn:publicid:IDN+example.net+authority+cm"));
        start.add(new SfaAuthority("urn:publicid:IDN+example.com:one+authority+cm"));
        start.add(new SfaAuthority("urn:publicid:IDN+example.com:aaaaa:bbbbb:three+authority+cm"));
        start.add(new SfaAuthority("urn:publicid:IDN+ex.com:c:d:e:four+authority+cm"));

        List<SfaAuthority> expected = new ArrayList<SfaAuthority>();
        expected.add(new SfaAuthority("urn:publicid:IDN+example.net+authority+cm"));
        expected.add(new SfaAuthority("urn:publicid:IDN+example.com:one+authority+cm"));
        expected.add(new SfaAuthority("urn:publicid:IDN+example.com:aaaaa:bbbbb:three+authority+cm"));
        expected.add(new SfaAuthority("urn:publicid:IDN+ex.com:c:d:e:four+authority+cm"));

        List<SfaAuthority> actual = AuthorityListModel.sortBySubAuthNumber(start);
        System.out.println("actual="+actual);

        assertThat(actual, is(start));
        assertThat(actual, equalTo(expected));
    }
}
