package be.iminds.ilabt.jfed.ui.javafx.probe_gui.command_arguments;

import be.iminds.ilabt.jfed.highlevel.model.CredentialInfo;
import be.iminds.ilabt.jfed.highlevel.model.ParameterHistoryModel;
import be.iminds.ilabt.jfed.ui.javafx.probe_gui.dialog.CredentialEditDialog;
import be.iminds.ilabt.jfed.ui.javafx.probe_gui.dialog.CredentialViewDialog;
import javafx.collections.ObservableList;
import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.scene.control.ComboBox;

import javax.inject.Inject;
import java.io.IOException;

/**
 * StringArgumentChooser
 */
public class CredentialArgumentChooser extends CommandArgumentChooser<CredentialInfo> {
    @FXML
    private ComboBox<CredentialInfo> comboBox;

    @Inject
    private final ParameterHistoryModel parameterHistoryModel;

    private final CredentialSubject credentialSubject;

    public CredentialArgumentChooser(ParameterHistoryModel parameterHistoryModel, CredentialSubject credentialSubject) {
        this.parameterHistoryModel = parameterHistoryModel;
        this.credentialSubject = credentialSubject;

        FXMLLoader fxmlLoader = new FXMLLoader(getClass().getResource("CredentialArgumentChooser.fxml"));
        fxmlLoader.setRoot(this);
        fxmlLoader.setController(this);

        try {
            fxmlLoader.load();

            assert parameterHistoryModel != null;
            assert comboBox != null;

            switch (credentialSubject) {
                case USER: {
                    comboBox.setItems(parameterHistoryModel.getUserCredentialsList());
                    break;
                }
                case SLICE: {
                    comboBox.setItems(parameterHistoryModel.getSliceCredentialsList());
                    break;
                }
                case ANY: {
                    comboBox.setItems(parameterHistoryModel.getAnyCredentialsList());
                    break;
                }
            }

            if (!comboBox.getItems().isEmpty())
                comboBox.getSelectionModel().selectLast();

            value = comboBox.getSelectionModel().selectedItemProperty();
            assert value != null;
        } catch (IOException exception) {
            throw new RuntimeException(exception);
        }
    }

    public void add() {
        ObservableList<CredentialInfo> creds = null;
        switch (credentialSubject) {
            case USER: {
                creds = parameterHistoryModel.getUserCredentialsList();
                break;
            }
            case SLICE: {
                creds = parameterHistoryModel.getSliceCredentialsList();
                break;
            }
            case ANY: {
                creds = parameterHistoryModel.getUncategorizedCredentiasList();
                break;
            }
        }
        CredentialInfo newCredentialInfo = CredentialEditDialog.showAddDialog();
        if (newCredentialInfo != null)
            creds.add(newCredentialInfo);
    }

    public void view() {
        CredentialInfo selected = comboBox.getSelectionModel().selectedItemProperty().get();
        if (selected != null)
            CredentialViewDialog.showDialog(selected);
    }
}
