package be.iminds.ilabt.jfed.ui.javafx.scanner;

import be.iminds.ilabt.jfed.lowlevel.authority.SfaAuthority;
import com.cathive.fx.guice.GuiceFXMLLoader;
import javafx.scene.Scene;
import javafx.stage.Modality;
import javafx.stage.Stage;
import javafx.stage.StageStyle;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.inject.Inject;
import javax.inject.Singleton;
import java.io.IOException;
import java.net.URL;

/**
 * User: twalcari
 * Date: 12/3/2014
 * Time: 12:06
 */
@Singleton
public class ScanResultPanelFactory {
    private static final Logger LOG = LoggerFactory.getLogger(ScanResultPanelFactory.class);
    private static final String SCAN_RESULT_PANEL_FXML = "ScanResultPanel.fxml";

    private final GuiceFXMLLoader loader;

    @Inject
    ScanResultPanelFactory(GuiceFXMLLoader loader) {
        this.loader = loader;
    }


    /**
     * @param auth
     * @param dialog
     * @return scanAgain
     */
    public boolean showScanResultPanel(SfaAuthority auth, boolean dialog) {
        Stage stage = new Stage();
        stage.setTitle("Server Scan Result");
        if (dialog) {
            stage.initStyle(StageStyle.UTILITY);
            stage.initModality(Modality.APPLICATION_MODAL);
        }

        try {
            URL location = getClass().getResource(SCAN_RESULT_PANEL_FXML);
            GuiceFXMLLoader.Result result = loader.load(location);

            ScanResultPanelController controller = result.getController();
            controller.setAuthority(auth);
            controller.setStage(stage);

            stage.setScene(new Scene(result.getRoot()));
            stage.sizeToScene();
            LOG.debug("Showing scanner panel");
            controller.showStandalone();
            stage.showAndWait();
            return controller.isScanAgain();
        } catch (IOException ex) {
            LOG.error("Error while loading ScanResultPanel", ex);
            return false;
        }
    }
}
