package be.iminds.ilabt.jfed.test;

import be.iminds.ilabt.jfed.lowlevel.JFedException;
import be.iminds.ilabt.jfed.lowlevel.ServerType;
import be.iminds.ilabt.jfed.lowlevel.api.AggregateManager2;
import be.iminds.ilabt.jfed.lowlevel.connection.BasicConnectionBuilder;
import be.iminds.ilabt.jfed.lowlevel.connection.SfaConnection;
import be.iminds.ilabt.jfed.util.KeyUtil;
import javafx.embed.swing.JFXPanel;
import org.slf4j.LoggerFactory;
import org.testng.annotations.*;

import java.net.MalformedURLException;
import java.net.URL;

import static org.hamcrest.MatcherAssert.assertThat;
import static org.hamcrest.Matchers.*;

//These tests only work when you have a test server running at localhost!
@Test(enabled=false)
public class JFedTestExampleWithDummyUser {
    JFedTesterBuilder testerBuilder;
    JFedTest jfed;

    @BeforeMethod
    public void setUp() throws Exception {
        //initialize JavaFX
        new JFXPanel();

        testerBuilder = new JFedTesterBuilder();

        //Use the built in test user.
        //   You have to trust the dummy user's authority for this to work, the certificate will be printed when the test runs.
        //   Or you can get it at JFedTesterBuilder.dummyAuthServerCertPem
        testerBuilder.setDummyUser();

        //Register the local AM server to test
        //  (this will automatically download and trust it's certificate, this is insecure, but that doesn't matter for a test like this)
        testerBuilder.registerAuthority("urn:publicid:IDN+dummy+authority+cm", 2, "https://localhost:8001");

        jfed = testerBuilder.build();
    }

    @AfterMethod
    public void tearDown() throws Exception {

    }

    @Test
    public void testLowLevelGetVersion() throws JFedException, MalformedURLException {
        SfaConnection con = (SfaConnection) jfed.connectionProvider.getConnectionByUrl(
                jfed.trustStore,
                jfed.user,
                jfed.am.getAmAuthority().getUrl(ServerType.GeniServerRole.AM, 2),
                new BasicConnectionBuilder.INSECURE_TRUSTALL_HandleUntrustedCallback(),
                new ServerType(ServerType.GeniServerRole.AM, 2),
                null);
        AggregateManager2 am2 = new AggregateManager2(jfed.logger, jfed.prefs);
        am2.getVersion(con);
    }

    @Test
    public void testGetVersion() throws JFedException {
        jfed.am.getVersion();
    }

    @Test
    public void testListResources() throws JFedException {
        String rspec = jfed.am.listResources(jfed.userCreds, false/*available*/);
        System.out.println("Received Advertisement RSpec: "+rspec);
    }
}
